@@ -1,4 +1,4 @@
-/* vim:set ts=4 sw=4 sts=4 et cindent: */
+/* vim:set ts=4 sw=2 sts=2 et cindent: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -14,8 +14,8 @@
 //
 
 #include "nsAuthSSPI.h"
+#include "nsComponentManagerUtils.h"
 #include "nsDNSService2.h"
-#include "nsIServiceManager.h"
 #include "nsIDNSService.h"
 #include "nsIDNSRecord.h"
 #include "nsMemory.h"
@@ -29,26 +29,26 @@
 #define SEC_SUCCESS(Status) ((Status) >= 0)
 
 #ifndef KERB_WRAP_NO_ENCRYPT
-#define KERB_WRAP_NO_ENCRYPT 0x80000001
+#  define KERB_WRAP_NO_ENCRYPT 0x80000001
 #endif
 
 #ifndef SECBUFFER_PADDING
-#define SECBUFFER_PADDING 9
+#  define SECBUFFER_PADDING 9
 #endif
 
 #ifndef SECBUFFER_STREAM
-#define SECBUFFER_STREAM 10
+#  define SECBUFFER_STREAM 10
 #endif
 
 //-----------------------------------------------------------------------------
 
-static const wchar_t *const pTypeName[] = {L"Kerberos", L"Negotiate", L"NTLM"};
+static const wchar_t* const pTypeName[] = {L"Kerberos", L"Negotiate", L"NTLM"};
 
 #ifdef DEBUG
-#define CASE_(_x) \
-  case _x:        \
-    return #_x;
-static const char *MapErrorCode(int rc) {
+#  define CASE_(_x) \
+    case _x:        \
+      return #_x;
+static const char* MapErrorCode(int rc) {
   switch (rc) {
     CASE_(SEC_E_OK)
     CASE_(SEC_I_CONTINUE_NEEDED)
@@ -68,7 +68,7 @@ static const char *MapErrorCode(int rc) {
   return "<unknown>";
 }
 #else
-#define MapErrorCode(_rc) ""
+#  define MapErrorCode(_rc) ""
 #endif
 
 //-----------------------------------------------------------------------------
@@ -89,7 +89,7 @@ static nsresult InitSSPI() {
 
 //-----------------------------------------------------------------------------
 
-nsresult nsAuthSSPI::MakeSN(const char *principal, nsCString &result) {
+nsresult nsAuthSSPI::MakeSN(const char* principal, nsCString& result) {
   nsresult rv;
 
   nsAutoCString buf(principal);
@@ -104,7 +104,7 @@ nsresult nsAuthSSPI::MakeSN(const char *principal, nsCString &result) {
       do_GetService(NS_DNSSERVICE_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
-  auto dns = static_cast<nsDNSService *>(dnsService.get());
+  auto dns = static_cast<nsDNSService*>(dnsService.get());
 
   // This could be expensive if our DNS cache cannot satisfy the request.
   // However, we should have at least hit the OS resolver once prior to
@@ -173,9 +173,9 @@ void nsAuthSSPI::Reset() {
 NS_IMPL_ISUPPORTS(nsAuthSSPI, nsIAuthModule)
 
 NS_IMETHODIMP
-nsAuthSSPI::Init(const char *serviceName, uint32_t serviceFlags,
-                 const char16_t *domain, const char16_t *username,
-                 const char16_t *password) {
+nsAuthSSPI::Init(const char* serviceName, uint32_t serviceFlags,
+                 const char16_t* domain, const char16_t* username,
+                 const char16_t* password) {
   LOG(("  nsAuthSSPI::Init\n"));
 
   mIsFirst = true;
@@ -193,9 +193,9 @@ nsAuthSSPI::Init(const char *serviceName, uint32_t serviceFlags,
     rv = InitSSPI();
     if (NS_FAILED(rv)) return rv;
   }
-  SEC_WCHAR *package;
+  SEC_WCHAR* package;
 
-  package = (SEC_WCHAR *)pTypeName[(int)mPackage];
+  package = (SEC_WCHAR*)pTypeName[(int)mPackage];
 
   if (mPackage == PACKAGE_TYPE_NTLM) {
     // (bug 535193) For NTLM, just use the uri host, do not do canonical host
@@ -229,7 +229,7 @@ nsAuthSSPI::Init(const char *serviceName, uint32_t serviceFlags,
   MS_TimeStamp useBefore;
 
   SEC_WINNT_AUTH_IDENTITY_W ai;
-  SEC_WINNT_AUTH_IDENTITY_W *pai = nullptr;
+  SEC_WINNT_AUTH_IDENTITY_W* pai = nullptr;
 
   // domain, username, and password will be null if nsHttpNTLMAuth's
   // ChallengeReceived returns false for identityInvalid. Use default
@@ -239,11 +239,11 @@ nsAuthSSPI::Init(const char *serviceName, uint32_t serviceFlags,
     mUsername.Assign(username);
     mPassword.Assign(password);
     mDomain.Assign(domain);
-    ai.Domain = reinterpret_cast<unsigned short *>(mDomain.BeginWriting());
+    ai.Domain = reinterpret_cast<unsigned short*>(mDomain.BeginWriting());
     ai.DomainLength = mDomain.Length();
-    ai.User = reinterpret_cast<unsigned short *>(mUsername.BeginWriting());
+    ai.User = reinterpret_cast<unsigned short*>(mUsername.BeginWriting());
     ai.UserLength = mUsername.Length();
-    ai.Password = reinterpret_cast<unsigned short *>(mPassword.BeginWriting());
+    ai.Password = reinterpret_cast<unsigned short*>(mPassword.BeginWriting());
     ai.PasswordLength = mPassword.Length();
     ai.Flags = SEC_WINNT_AUTH_IDENTITY_UNICODE;
     pai = &ai;
@@ -271,8 +271,8 @@ nsAuthSSPI::Init(const char *serviceName, uint32_t serviceFlags,
 // certificate (when available) in the first call of the function. The
 // second time these arguments hold an input token.
 NS_IMETHODIMP
-nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
-                         void **outToken, uint32_t *outTokenLen) {
+nsAuthSSPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
+                         void** outToken, uint32_t* outTokenLen) {
   // String for end-point bindings.
   const char end_point[] = "tls-server-end-point:";
   const int end_point_length = sizeof(end_point) - 1;
@@ -283,12 +283,12 @@ nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
   MS_TimeStamp ignored;
 
   DWORD ctxAttr, ctxReq = 0;
-  CtxtHandle *ctxIn;
+  CtxtHandle* ctxIn;
   SecBufferDesc ibd, obd;
   // Optional second input buffer for the CBT (Channel Binding Token)
   SecBuffer ib[2], ob;
   // Pointer to the block of memory that stores the CBT
-  char *sspi_cbt = nullptr;
+  char* sspi_cbt = nullptr;
   SEC_CHANNEL_BINDINGS pendpoint_binding;
 
   LOG(("entering nsAuthSSPI::GetNextToken()\n"));
@@ -308,7 +308,6 @@ nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
       mIsFirst = false;
       mCertDERLength = inTokenLen;
       mCertDERData = moz_xmalloc(inTokenLen);
-      if (!mCertDERData) return NS_ERROR_OUT_OF_MEMORY;
       memcpy(mCertDERData, inToken, inTokenLen);
 
       // We are starting a new authentication sequence.
@@ -349,13 +348,10 @@ nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
         ib[ibd.cBuffers].cbBuffer = pendpoint_binding.cbApplicationDataLength +
                                     pendpoint_binding.dwApplicationDataOffset;
 
-        sspi_cbt = (char *)moz_xmalloc(ib[ibd.cBuffers].cbBuffer);
-        if (!sspi_cbt) {
-          return NS_ERROR_OUT_OF_MEMORY;
-        }
+        sspi_cbt = (char*)moz_xmalloc(ib[ibd.cBuffers].cbBuffer);
 
         // Helper to write in the memory block that stores the CBT
-        char *sspi_cbt_ptr = sspi_cbt;
+        char* sspi_cbt_ptr = sspi_cbt;
 
         ib[ibd.cBuffers].pvBuffer = sspi_cbt;
         ibd.cBuffers++;
@@ -376,7 +372,7 @@ nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
         crypto = do_CreateInstance(NS_CRYPTO_HASH_CONTRACTID, &rv);
         if (NS_SUCCEEDED(rv)) rv = crypto->Init(nsICryptoHash::SHA256);
         if (NS_SUCCEEDED(rv))
-          rv = crypto->Update((unsigned char *)mCertDERData, mCertDERLength);
+          rv = crypto->Update((unsigned char*)mCertDERData, mCertDERLength);
         if (NS_SUCCEEDED(rv)) rv = crypto->Finish(false, hashString);
         if (NS_FAILED(rv)) {
           free(mCertDERData);
@@ -400,7 +396,7 @@ nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
       // We always need this SECBUFFER.
       ib[ibd.cBuffers].BufferType = SECBUFFER_TOKEN;
       ib[ibd.cBuffers].cbBuffer = inTokenLen;
-      ib[ibd.cBuffers].pvBuffer = (void *)inToken;
+      ib[ibd.cBuffers].pvBuffer = (void*)inToken;
       ibd.cBuffers++;
       ctxIn = &mCtxt;
     }
@@ -423,14 +419,10 @@ nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
   ob.BufferType = SECBUFFER_TOKEN;
   ob.cbBuffer = mMaxTokenLen;
   ob.pvBuffer = moz_xmalloc(ob.cbBuffer);
-  if (!ob.pvBuffer) {
-    if (sspi_cbt) free(sspi_cbt);
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
   memset(ob.pvBuffer, 0, ob.cbBuffer);
 
   NS_ConvertUTF8toUTF16 wSN(mServiceName);
-  SEC_WCHAR *sn = (SEC_WCHAR *)wSN.get();
+  SEC_WCHAR* sn = (SEC_WCHAR*)wSN.get();
 
   rc = (sspi->InitializeSecurityContextW)(
       &mCred, ctxIn, sn, ctxReq, 0, SECURITY_NATIVE_DREP,
@@ -462,8 +454,8 @@ nsAuthSSPI::GetNextToken(const void *inToken, uint32_t inTokenLen,
 }
 
 NS_IMETHODIMP
-nsAuthSSPI::Unwrap(const void *inToken, uint32_t inTokenLen, void **outToken,
-                   uint32_t *outTokenLen) {
+nsAuthSSPI::Unwrap(const void* inToken, uint32_t inTokenLen, void** outToken,
+                   uint32_t* outTokenLen) {
   SECURITY_STATUS rc;
   SecBufferDesc ibd;
   SecBuffer ib[2];
@@ -476,7 +468,6 @@ nsAuthSSPI::Unwrap(const void *inToken, uint32_t inTokenLen, void **outToken,
   ib[0].BufferType = SECBUFFER_STREAM;
   ib[0].cbBuffer = inTokenLen;
   ib[0].pvBuffer = moz_xmalloc(ib[0].cbBuffer);
-  if (!ib[0].pvBuffer) return NS_ERROR_OUT_OF_MEMORY;
 
   memcpy(ib[0].pvBuffer, inToken, inTokenLen);
 
@@ -496,9 +487,8 @@ nsAuthSSPI::Unwrap(const void *inToken, uint32_t inTokenLen, void **outToken,
     if (ib[0].pvBuffer == ib[1].pvBuffer) {
       *outToken = ib[1].pvBuffer;
     } else {
-      *outToken = nsMemory::Clone(ib[1].pvBuffer, ib[1].cbBuffer);
+      *outToken = moz_xmemdup(ib[1].pvBuffer, ib[1].cbBuffer);
       free(ib[0].pvBuffer);
-      if (!*outToken) return NS_ERROR_OUT_OF_MEMORY;
     }
     *outTokenLen = ib[1].cbBuffer;
   } else
@@ -526,8 +516,8 @@ class secBuffers {
 };
 
 NS_IMETHODIMP
-nsAuthSSPI::Wrap(const void *inToken, uint32_t inTokenLen, bool confidential,
-                 void **outToken, uint32_t *outTokenLen) {
+nsAuthSSPI::Wrap(const void* inToken, uint32_t inTokenLen, bool confidential,
+                 void** outToken, uint32_t* outTokenLen) {
   SECURITY_STATUS rc;
 
   SecBufferDesc ibd;
@@ -547,15 +537,11 @@ nsAuthSSPI::Wrap(const void *inToken, uint32_t inTokenLen, bool confidential,
   bufs.ib[0].BufferType = SECBUFFER_TOKEN;
   bufs.ib[0].pvBuffer = moz_xmalloc(sizes.cbSecurityTrailer);
 
-  if (!bufs.ib[0].pvBuffer) return NS_ERROR_OUT_OF_MEMORY;
-
   // APP Data
   bufs.ib[1].BufferType = SECBUFFER_DATA;
   bufs.ib[1].pvBuffer = moz_xmalloc(inTokenLen);
   bufs.ib[1].cbBuffer = inTokenLen;
 
-  if (!bufs.ib[1].pvBuffer) return NS_ERROR_OUT_OF_MEMORY;
-
   memcpy(bufs.ib[1].pvBuffer, inToken, inTokenLen);
 
   // SSPI
@@ -563,18 +549,14 @@ nsAuthSSPI::Wrap(const void *inToken, uint32_t inTokenLen, bool confidential,
   bufs.ib[2].cbBuffer = sizes.cbBlockSize;
   bufs.ib[2].pvBuffer = moz_xmalloc(bufs.ib[2].cbBuffer);
 
-  if (!bufs.ib[2].pvBuffer) return NS_ERROR_OUT_OF_MEMORY;
-
   rc = (sspi->EncryptMessage)(&mCtxt, confidential ? 0 : KERB_WRAP_NO_ENCRYPT,
                               &ibd, 0);
 
   if (SEC_SUCCESS(rc)) {
     int len = bufs.ib[0].cbBuffer + bufs.ib[1].cbBuffer + bufs.ib[2].cbBuffer;
-    char *p = (char *)moz_xmalloc(len);
+    char* p = (char*)moz_xmalloc(len);
 
-    if (!p) return NS_ERROR_OUT_OF_MEMORY;
-
-    *outToken = (void *)p;
+    *outToken = (void*)p;
     *outTokenLen = len;
 
     memcpy(p, bufs.ib[0].pvBuffer, bufs.ib[0].cbBuffer);
