@@ -8,7 +8,7 @@
 #include "nsCOMPtr.h"
 #include "nsTArray.h"
 
-class nsISpellChecker;
+class mozSpellChecker;
 
 namespace mozilla {
 
@@ -21,20 +21,20 @@ class RemoteSpellcheckEngineParent : public PRemoteSpellcheckEngineParent {
   virtual void ActorDestroy(ActorDestroyReason aWhy) override;
 
   virtual mozilla::ipc::IPCResult RecvSetDictionary(const nsString& aDictionary,
-                                                    bool* success) override;
+                                                    bool* success);
 
   virtual mozilla::ipc::IPCResult RecvSetDictionaryFromList(
-      nsTArray<nsString>&& aList, const intptr_t& aPromiseId) override;
+      nsTArray<nsString>&& aList, SetDictionaryFromListResolver&& aResolve);
 
-  virtual mozilla::ipc::IPCResult RecvCheck(const nsString& aWord,
-                                            bool* aIsMisspelled) override;
+  virtual mozilla::ipc::IPCResult RecvCheckAsync(nsTArray<nsString>&& aWord,
+                                                 CheckAsyncResolver&& aResolve);
 
   virtual mozilla::ipc::IPCResult RecvCheckAndSuggest(
       const nsString& aWord, bool* aIsMisspelled,
-      InfallibleTArray<nsString>* aSuggestions) override;
+      nsTArray<nsString>* aSuggestions);
 
  private:
-  nsCOMPtr<nsISpellChecker> mSpellChecker;
+  RefPtr<mozSpellChecker> mSpellChecker;
 };
 
 }  // namespace mozilla
