@@ -6,6 +6,8 @@
 #include "LookupCacheV4.h"
 #include "HashStore.h"
 #include "mozilla/Unused.h"
+#include "nsCheckSummedOutputStream.h"
+#include "crc32c.h"
 #include <string>
 
 // MOZ_LOG=UrlClassifierDbService:5
@@ -21,8 +23,12 @@ namespace mozilla {
 namespace safebrowsing {
 
 const int LookupCacheV4::VER = 4;
+const uint32_t LookupCacheV4::VLPSET_MAGIC = 0x36044a35;
+const uint32_t LookupCacheV4::VLPSET_VERSION = 1;
 const uint32_t LookupCacheV4::MAX_METADATA_VALUE_LENGTH = 256;
 
+////////////////////////////////////////////////////////////////////////
+
 // Prefixes coming from updates and VLPrefixSet are both stored in the HashTable
 // where the (key, value) pair is a prefix size and a lexicographic-sorted
 // string. The difference is prefixes from updates use std:string(to avoid
@@ -32,13 +38,12 @@ const uint32_t LookupCacheV4::MAX_METADATA_VALUE_LENGTH = 256;
 class VLPrefixSet {
  public:
   explicit VLPrefixSet(const PrefixStringMap& aMap);
-  explicit VLPrefixSet(const TableUpdateV4::PrefixStdStringMap& aMap);
 
   // This function will merge the prefix map in VLPrefixSet to aPrefixMap.
   void Merge(PrefixStringMap& aPrefixMap);
 
   // Find the smallest string from the map in VLPrefixSet.
-  bool GetSmallestPrefix(nsDependentCSubstring& aOutString);
+  bool GetSmallestPrefix(nsACString& aOutString) const;
 
   // Return the number of prefixes in the map
   uint32_t Count() const { return mCount; }
@@ -48,17 +53,37 @@ class VLPrefixSet {
   // a |pos| variable to indicate which substring we are pointing to right now.
   // |pos| increases each time GetSmallestPrefix finds the smallest string.
   struct PrefixString {
-    PrefixString(const nsACString& aStr, uint32_t aSize) : pos(0), size(aSize) {
-      data.Rebind(aStr.BeginReading(), aStr.Length());
+    PrefixString(const nsACString& aStr, uint32_t aSize)
+        : data(aStr), pos(0), size(aSize) {
+      MOZ_ASSERT(data.Length() % size == 0,
+                 "PrefixString length must be a multiple of the prefix size.");
     }
 
-    const char* get() {
-      return pos < data.Length() ? data.BeginReading() + pos : nullptr;
+    void getRemainingString(nsACString& out) {
+      MOZ_ASSERT(out.IsEmpty());
+      if (remaining() > 0) {
+        out = Substring(data, pos);
+      }
+    }
+    void getPrefix(nsACString& out) {
+      MOZ_ASSERT(out.IsEmpty());
+      if (remaining() >= size) {
+        out = Substring(data, pos, size);
+      } else {
+        MOZ_ASSERT(remaining() == 0,
+                   "Remaining bytes but not enough for a (size)-byte prefix.");
+      }
+    }
+    void next() {
+      pos += size;
+      MOZ_ASSERT(pos <= data.Length());
+    }
+    uint32_t remaining() {
+      return data.Length() - pos;
+      MOZ_ASSERT(pos <= data.Length());
     }
-    void next() { pos += size; }
-    uint32_t remaining() { return data.Length() - pos; }
 
-    nsDependentCSubstring data;
+    nsCString data;
     uint32_t pos;
     uint32_t size;
   };
@@ -67,14 +92,6 @@ class VLPrefixSet {
   uint32_t mCount;
 };
 
-nsresult LookupCacheV4::Init() {
-  mVLPrefixSet = new VariableLengthPrefixSet();
-  nsresult rv = mVLPrefixSet->Init(mTableName);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
 nsresult LookupCacheV4::Has(const Completion& aCompletion, bool* aHas,
                             uint32_t* aMatchLength, bool* aConfirmed) {
   *aHas = *aConfirmed = false;
@@ -84,44 +101,44 @@ nsresult LookupCacheV4::Has(const Completion& aCompletion, bool* aHas,
   nsDependentCSubstring fullhash;
   fullhash.Rebind((const char*)aCompletion.buf, COMPLETE_SIZE);
 
-  nsresult rv = mVLPrefixSet->Matches(fullhash, &length);
+  // It is tricky that we use BigEndian read for V4 while use
+  // Completion.ToUint32 for V2. This is because in V2, prefixes are converted
+  // to integers and then sorted internally. In V4, prefixes recieved are
+  // already lexicographical order sorted, so when we manipulate these prefixes
+  // with integer form, we always use big endian so prefixes remain the same
+  // order.
+  uint32_t prefix = BigEndian::readUint32(
+      reinterpret_cast<const uint32_t*>(fullhash.BeginReading()));
+
+  nsresult rv = mVLPrefixSet->Matches(prefix, fullhash, &length);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  MOZ_ASSERT(length == 0 || (length >= PREFIX_SIZE && length <= COMPLETE_SIZE));
-
-  *aHas = length >= PREFIX_SIZE;
-  *aMatchLength = length;
-
-  if (LOG_ENABLED()) {
-    uint32_t prefix = aCompletion.ToUint32();
-    LOG(("Probe in V4 %s: %X, found %d, complete %d", mTableName.get(), prefix,
-         *aHas, length == COMPLETE_SIZE));
-  }
-
-  // Check if fullhash match any prefix in the local database
-  if (!(*aHas)) {
+  if (length == 0) {
     return NS_OK;
   }
 
+  MOZ_ASSERT(length >= PREFIX_SIZE && length <= COMPLETE_SIZE);
+
+  // For V4, We don't set |aConfirmed| to true even if we found a match
+  // for 32-bytes prefix. |aConfirmed| is only set if a match is found in cache.
+  *aHas = true;
+  *aMatchLength = length;
+
   // Even though V4 supports variable-length prefix, we always send 4-bytes for
   // completion (Bug 1323953). This means cached prefix length is also 4-bytes.
   return CheckCache(aCompletion, aHas, aConfirmed);
 }
 
-bool LookupCacheV4::IsEmpty() {
-  bool isEmpty;
-  mVLPrefixSet->IsEmpty(&isEmpty);
-  return isEmpty;
-}
-
 nsresult LookupCacheV4::Build(PrefixStringMap& aPrefixMap) {
   Telemetry::AutoTimer<Telemetry::URLCLASSIFIER_VLPS_CONSTRUCT_TIME> timer;
 
   nsresult rv = mVLPrefixSet->SetPrefixes(aPrefixMap);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
   mPrimed = true;
 
-  return rv;
+  return NS_OK;
 }
 
 nsresult LookupCacheV4::GetPrefixes(PrefixStringMap& aPrefixMap) {
@@ -135,62 +152,154 @@ nsresult LookupCacheV4::GetPrefixes(PrefixStringMap& aPrefixMap) {
 
 nsresult LookupCacheV4::GetFixedLengthPrefixes(
     FallibleTArray<uint32_t>& aPrefixes) {
-  return mVLPrefixSet->GetFixedLengthPrefixes(aPrefixes);
+  return mVLPrefixSet->GetFixedLengthPrefixes(&aPrefixes, nullptr);
 }
 
-nsresult LookupCacheV4::ClearPrefixes() {
-  // Clear by seting a empty map
-  PrefixStringMap map;
-  return mVLPrefixSet->SetPrefixes(map);
-}
-
-nsresult LookupCacheV4::StoreToFile(nsIFile* aFile) {
-  return mVLPrefixSet->StoreToFile(aFile);
-}
-
-nsresult LookupCacheV4::LoadFromFile(nsIFile* aFile) {
-  nsresult rv = mVLPrefixSet->LoadFromFile(aFile);
-  if (NS_FAILED(rv)) {
+nsresult LookupCacheV4::ClearLegacyFile() {
+  nsCOMPtr<nsIFile> file;
+  nsresult rv = mStoreDirectory->Clone(getter_AddRefs(file));
+  if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  nsCString state, checksum;
-  rv = LoadMetadata(state, checksum);
-  Telemetry::Accumulate(Telemetry::URLCLASSIFIER_VLPS_METADATA_CORRUPT,
-                        rv == NS_ERROR_FILE_CORRUPTED);
-  if (NS_FAILED(rv)) {
+  rv = file->AppendNative(mTableName + NS_LITERAL_CSTRING(".pset"));
+  if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  rv = VerifyChecksum(checksum);
-  Telemetry::Accumulate(Telemetry::URLCLASSIFIER_VLPS_LOAD_CORRUPT,
-                        rv == NS_ERROR_FILE_CORRUPTED);
+  bool exists;
+  rv = file->Exists(&exists);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
 
-  return rv;
+  if (exists) {
+    rv = file->Remove(false);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+
+    LOG(("[%s] Old PrefixSet is successfully removed!", mTableName.get()));
+  }
+
+  return NS_OK;
 }
 
-size_t LookupCacheV4::SizeOfPrefixSet() {
-  return mVLPrefixSet->SizeOfIncludingThis(moz_malloc_size_of);
+nsresult LookupCacheV4::LoadLegacyFile() {
+  nsCOMPtr<nsIFile> file;
+  nsresult rv = mStoreDirectory->Clone(getter_AddRefs(file));
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  rv = file->AppendNative(mTableName + NS_LITERAL_CSTRING(".pset"));
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  bool exists;
+  rv = file->Exists(&exists);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!exists) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsIInputStream> localInFile;
+  rv = NS_NewLocalFileInputStream(getter_AddRefs(localInFile), file,
+                                  PR_RDONLY | nsIFile::OS_READAHEAD);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  // Calculate how big the file is, make sure our read buffer isn't bigger
+  // than the file itself which is just wasting memory.
+  int64_t fileSize;
+  rv = file->GetFileSize(&fileSize);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  if (fileSize < 0 || fileSize > UINT32_MAX) {
+    return NS_ERROR_FAILURE;
+  }
+
+  uint32_t bufferSize =
+      std::min<uint32_t>(static_cast<uint32_t>(fileSize), MAX_BUFFER_SIZE);
+
+  // Convert to buffered stream
+  nsCOMPtr<nsIInputStream> in;
+  rv = NS_NewBufferedInputStream(getter_AddRefs(in), localInFile.forget(),
+                                 bufferSize);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  // Load data
+  rv = mVLPrefixSet->LoadPrefixes(in);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  mPrimed = true;
+
+  LOG(("[%s] Loading Legacy PrefixSet successful", mTableName.get()));
+  return NS_OK;
+}
+
+void LookupCacheV4::GetHeader(Header& aHeader) {
+  aHeader.magic = LookupCacheV4::VLPSET_MAGIC;
+  aHeader.version = LookupCacheV4::VLPSET_VERSION;
+}
+
+nsresult LookupCacheV4::SanityCheck(const Header& aHeader) {
+  if (aHeader.magic != LookupCacheV4::VLPSET_MAGIC) {
+    return NS_ERROR_FILE_CORRUPTED;
+  }
+
+  if (aHeader.version != LookupCacheV4::VLPSET_VERSION) {
+    return NS_ERROR_FAILURE;
+  }
+
+  return NS_OK;
+}
+
+nsCString LookupCacheV4::GetPrefixSetSuffix() const {
+  return NS_LITERAL_CSTRING(".vlpset");
 }
 
 static nsresult AppendPrefixToMap(PrefixStringMap& prefixes,
-                                  nsDependentCSubstring& prefix) {
+                                  const nsACString& prefix) {
   uint32_t len = prefix.Length();
+  MOZ_ASSERT(len >= PREFIX_SIZE && len <= COMPLETE_SIZE);
   if (!len) {
     return NS_OK;
   }
 
   nsCString* prefixString = prefixes.LookupOrAdd(len);
-  if (!prefixString->Append(prefix.BeginReading(), len, fallible)) {
+  if (!prefixString->Append(prefix, fallible)) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
   return NS_OK;
 }
 
+static nsresult InitCrypto(nsCOMPtr<nsICryptoHash>& aCrypto) {
+  nsresult rv;
+  aCrypto = do_CreateInstance(NS_CRYPTO_HASH_CONTRACTID, &rv);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  rv = aCrypto->Init(nsICryptoHash::SHA256);
+  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "InitCrypto failed");
+
+  return rv;
+}
+
 // Read prefix into a buffer and also update the hash which
-// keeps track of the checksum
-static void UpdateChecksum(nsICryptoHash* aCrypto, const nsACString& aPrefix) {
+// keeps track of the sha256 hash
+static void UpdateSHA256(nsICryptoHash* aCrypto, const nsACString& aPrefix) {
   MOZ_ASSERT(aCrypto);
   aCrypto->Update(
       reinterpret_cast<uint8_t*>(const_cast<char*>(aPrefix.BeginReading())),
@@ -199,7 +308,7 @@ static void UpdateChecksum(nsICryptoHash* aCrypto, const nsACString& aPrefix) {
 
 // Please see https://bug1287058.bmoattachments.org/attachment.cgi?id=8795366
 // for detail about partial update algorithm.
-nsresult LookupCacheV4::ApplyUpdate(TableUpdateV4* aTableUpdate,
+nsresult LookupCacheV4::ApplyUpdate(RefPtr<TableUpdateV4> aTableUpdate,
                                     PrefixStringMap& aInputMap,
                                     PrefixStringMap& aOutputMap) {
   MOZ_ASSERT(aOutputMap.IsEmpty());
@@ -221,13 +330,13 @@ nsresult LookupCacheV4::ApplyUpdate(TableUpdateV4* aTableUpdate,
   // |removalIndex| is the current index of RemovalIndiceArray.
   // |numOldPrefixPicked| is used to record how many prefixes we picked from the
   // old map.
-  TableUpdateV4::RemovalIndiceArray& removalArray =
+  const TableUpdateV4::RemovalIndiceArray& removalArray =
       aTableUpdate->RemovalIndices();
   uint32_t removalIndex = 0;
   int32_t numOldPrefixPicked = -1;
 
-  nsDependentCSubstring smallestOldPrefix;
-  nsDependentCSubstring smallestAddPrefix;
+  nsAutoCString smallestOldPrefix;
+  nsAutoCString smallestAddPrefix;
 
   bool isOldMapEmpty = false, isAddMapEmpty = false;
 
@@ -282,7 +391,7 @@ nsresult LookupCacheV4::ApplyUpdate(TableUpdateV4* aTableUpdate,
           return rv;
         }
 
-        UpdateChecksum(crypto, smallestOldPrefix);
+        UpdateSHA256(crypto, smallestOldPrefix);
       }
       smallestOldPrefix.SetLength(0);
     } else {
@@ -291,7 +400,7 @@ nsresult LookupCacheV4::ApplyUpdate(TableUpdateV4* aTableUpdate,
         return rv;
       }
 
-      UpdateChecksum(crypto, smallestAddPrefix);
+      UpdateSHA256(crypto, smallestAddPrefix);
       smallestAddPrefix.SetLength(0);
     }
   }
@@ -310,20 +419,24 @@ nsresult LookupCacheV4::ApplyUpdate(TableUpdateV4* aTableUpdate,
     return NS_ERROR_UC_UPDATE_WRONG_REMOVAL_INDICES;
   }
 
-  nsAutoCString checksum;
-  crypto->Finish(false, checksum);
-  if (aTableUpdate->Checksum().IsEmpty()) {
-    LOG(("Update checksum missing."));
+  // Prefixes and removal indice from update is no longer required
+  // after merging the data with local prefixes.
+  aTableUpdate->Clear();
+
+  nsAutoCString sha256;
+  crypto->Finish(false, sha256);
+  if (aTableUpdate->SHA256().IsEmpty()) {
+    LOG(("Update sha256 hash missing."));
     Telemetry::Accumulate(
         Telemetry::URLCLASSIFIER_UPDATE_ERROR, mProvider,
         NS_ERROR_GET_CODE(NS_ERROR_UC_UPDATE_MISSING_CHECKSUM));
 
-    // Generate our own checksum to tableUpdate to ensure there is always
+    // Generate our own sha256 to tableUpdate to ensure there is always
     // checksum in .metadata
-    std::string stdChecksum(checksum.BeginReading(), checksum.Length());
-    aTableUpdate->NewChecksum(stdChecksum);
-  } else if (aTableUpdate->Checksum() != checksum) {
-    LOG(("Checksum mismatch after applying partial update"));
+    std::string stdSha256(sha256.BeginReading(), sha256.Length());
+    aTableUpdate->SetSHA256(stdSha256);
+  } else if (aTableUpdate->SHA256() != sha256) {
+    LOG(("SHA256 hash mismatch after applying partial update"));
     return NS_ERROR_UC_UPDATE_CHECKSUM_MISMATCH;
   }
 
@@ -337,139 +450,8 @@ nsresult LookupCacheV4::AddFullHashResponseToCache(
   return NS_OK;
 }
 
-nsresult LookupCacheV4::InitCrypto(nsCOMPtr<nsICryptoHash>& aCrypto) {
-  nsresult rv;
-  aCrypto = do_CreateInstance(NS_CRYPTO_HASH_CONTRACTID, &rv);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  rv = aCrypto->Init(nsICryptoHash::SHA256);
-  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "InitCrypto failed");
-
-  return rv;
-}
-
-nsresult LookupCacheV4::VerifyChecksum(const nsACString& aChecksum) {
-  nsCOMPtr<nsICryptoHash> crypto;
-  nsresult rv = InitCrypto(crypto);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  PrefixStringMap map;
-  mVLPrefixSet->GetPrefixes(map);
-
-  VLPrefixSet loadPSet(map);
-  uint32_t index = loadPSet.Count() + 1;
-  for (; index > 0; index--) {
-    nsDependentCSubstring prefix;
-    if (!loadPSet.GetSmallestPrefix(prefix)) {
-      break;
-    }
-    UpdateChecksum(crypto, prefix);
-  }
-
-  nsAutoCString checksum;
-  crypto->Finish(false, checksum);
-
-  if (checksum != aChecksum) {
-    LOG(("Checksum mismatch when loading prefixes from file."));
-    return NS_ERROR_FILE_CORRUPTED;
-  }
-
-  return NS_OK;
-}
-
-//////////////////////////////////////////////////////////////////////////
-// A set of lightweight functions for reading/writing value from/to file.
-
-namespace {
-
-template <typename T>
-struct ValueTraits {
-  static_assert(sizeof(T) <= LookupCacheV4::MAX_METADATA_VALUE_LENGTH,
-                "LookupCacheV4::MAX_METADATA_VALUE_LENGTH is too small.");
-  static uint32_t Length(const T& aValue) { return sizeof(T); }
-  static char* WritePtr(T& aValue, uint32_t aLength) { return (char*)&aValue; }
-  static const char* ReadPtr(const T& aValue) { return (char*)&aValue; }
-  static bool IsFixedLength() { return true; }
-};
-
-template <>
-struct ValueTraits<nsACString> {
-  static bool IsFixedLength() { return false; }
-
-  static uint32_t Length(const nsACString& aValue) { return aValue.Length(); }
-
-  static char* WritePtr(nsACString& aValue, uint32_t aLength) {
-    aValue.SetLength(aLength);
-    return aValue.BeginWriting();
-  }
-
-  static const char* ReadPtr(const nsACString& aValue) {
-    return aValue.BeginReading();
-  }
-};
-
-template <typename T>
-static nsresult WriteValue(nsIOutputStream* aOutputStream, const T& aValue) {
-  uint32_t writeLength = ValueTraits<T>::Length(aValue);
-  MOZ_ASSERT(writeLength <= LookupCacheV4::MAX_METADATA_VALUE_LENGTH,
-             "LookupCacheV4::MAX_METADATA_VALUE_LENGTH is too small.");
-  if (!ValueTraits<T>::IsFixedLength()) {
-    // We need to write out the variable value length.
-    nsresult rv = WriteValue(aOutputStream, writeLength);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  // Write out the value.
-  auto valueReadPtr = ValueTraits<T>::ReadPtr(aValue);
-  uint32_t written;
-  nsresult rv = aOutputStream->Write(valueReadPtr, writeLength, &written);
-  if (NS_FAILED(rv) || written != writeLength) {
-    LOG(("Failed to write the value."));
-    return NS_FAILED(rv) ? rv : NS_ERROR_FAILURE;
-  }
-
-  return rv;
-}
-
-template <typename T>
-static nsresult ReadValue(nsIInputStream* aInputStream, T& aValue) {
-  nsresult rv;
-
-  uint32_t readLength;
-  if (ValueTraits<T>::IsFixedLength()) {
-    readLength = ValueTraits<T>::Length(aValue);
-  } else {
-    // Read the variable value length from file.
-    nsresult rv = ReadValue(aInputStream, readLength);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  // Sanity-check the readLength in case of disk corruption
-  // (see bug 1433636).
-  if (readLength > LookupCacheV4::MAX_METADATA_VALUE_LENGTH) {
-    return NS_ERROR_FILE_CORRUPTED;
-  }
-
-  // Read the value.
-  uint32_t read;
-  auto valueWritePtr = ValueTraits<T>::WritePtr(aValue, readLength);
-  rv = aInputStream->Read(valueWritePtr, readLength, &read);
-  if (NS_FAILED(rv) || read != readLength) {
-    LOG(("Failed to read the value."));
-    return NS_FAILED(rv) ? rv : NS_ERROR_FAILURE;
-  }
-
-  return rv;
-}
-
-}  // end of unnamed namespace.
-////////////////////////////////////////////////////////////////////////
-
-nsresult LookupCacheV4::WriteMetadata(TableUpdateV4* aTableUpdate) {
+nsresult LookupCacheV4::WriteMetadata(
+    RefPtr<const TableUpdateV4> aTableUpdate) {
   NS_ENSURE_ARG_POINTER(aTableUpdate);
   if (nsUrlClassifierDBService::ShutdownHasStarted()) {
     return NS_ERROR_ABORT;
@@ -485,30 +467,20 @@ nsresult LookupCacheV4::WriteMetadata(TableUpdateV4* aTableUpdate) {
   nsCOMPtr<nsIOutputStream> outputStream;
   rv = NS_NewLocalFileOutputStream(getter_AddRefs(outputStream), metaFile,
                                    PR_WRONLY | PR_TRUNCATE | PR_CREATE_FILE);
-  if (!NS_SUCCEEDED(rv)) {
-    LOG(("Unable to create file to store metadata."));
-    return rv;
-  }
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Write the state.
   rv = WriteValue(outputStream, aTableUpdate->ClientState());
-  if (NS_FAILED(rv)) {
-    LOG(("Failed to write the list state."));
-    return rv;
-  }
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  // Write the checksum.
-  rv = WriteValue(outputStream, aTableUpdate->Checksum());
-  if (NS_FAILED(rv)) {
-    LOG(("Failed to write the list checksum."));
-    return rv;
-  }
+  // Write the SHA256 hash.
+  rv = WriteValue(outputStream, aTableUpdate->SHA256());
+  NS_ENSURE_SUCCESS(rv, rv);
 
   return rv;
 }
 
-nsresult LookupCacheV4::LoadMetadata(nsACString& aState,
-                                     nsACString& aChecksum) {
+nsresult LookupCacheV4::LoadMetadata(nsACString& aState, nsACString& aSHA256) {
   nsCOMPtr<nsIFile> metaFile;
   nsresult rv = mStoreDirectory->Clone(getter_AddRefs(metaFile));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -531,10 +503,10 @@ nsresult LookupCacheV4::LoadMetadata(nsACString& aState,
     return rv;
   }
 
-  // Read the checksum.
-  rv = ReadValue(localInFile, aChecksum);
+  // Read the SHA256 hash.
+  rv = ReadValue(localInFile, aSHA256);
   if (NS_FAILED(rv)) {
-    LOG(("Failed to read checksum."));
+    LOG(("Failed to read SHA256 hash."));
     return rv;
   }
 
@@ -544,49 +516,48 @@ nsresult LookupCacheV4::LoadMetadata(nsACString& aState,
 VLPrefixSet::VLPrefixSet(const PrefixStringMap& aMap) : mCount(0) {
   for (auto iter = aMap.ConstIter(); !iter.Done(); iter.Next()) {
     uint32_t size = iter.Key();
+    MOZ_ASSERT(iter.Data()->Length() % size == 0,
+               "PrefixString must be a multiple of the prefix size.");
     mMap.Put(size, new PrefixString(*iter.Data(), size));
     mCount += iter.Data()->Length() / size;
   }
 }
 
-VLPrefixSet::VLPrefixSet(const TableUpdateV4::PrefixStdStringMap& aMap)
-    : mCount(0) {
-  for (auto iter = aMap.ConstIter(); !iter.Done(); iter.Next()) {
-    uint32_t size = iter.Key();
-    mMap.Put(size, new PrefixString(iter.Data()->GetPrefixString(), size));
-    mCount += iter.Data()->GetPrefixString().Length() / size;
-  }
-}
-
 void VLPrefixSet::Merge(PrefixStringMap& aPrefixMap) {
   for (auto iter = mMap.ConstIter(); !iter.Done(); iter.Next()) {
     nsCString* prefixString = aPrefixMap.LookupOrAdd(iter.Key());
-    PrefixString* str = iter.Data();
+    PrefixString* str = iter.UserData();
 
-    if (str->get()) {
-      prefixString->Append(str->get(), str->remaining());
+    nsAutoCString remainingString;
+    str->getRemainingString(remainingString);
+    if (!remainingString.IsEmpty()) {
+      MOZ_ASSERT(remainingString.Length() == str->remaining());
+      prefixString->Append(remainingString);
     }
   }
 }
 
-bool VLPrefixSet::GetSmallestPrefix(nsDependentCSubstring& aOutString) {
+bool VLPrefixSet::GetSmallestPrefix(nsACString& aOutString) const {
   PrefixString* pick = nullptr;
   for (auto iter = mMap.ConstIter(); !iter.Done(); iter.Next()) {
-    PrefixString* str = iter.Data();
+    PrefixString* str = iter.UserData();
 
-    if (!str->get()) {
+    if (str->remaining() <= 0) {
       continue;
     }
 
     if (aOutString.IsEmpty()) {
-      aOutString.Rebind(str->get(), iter.Key());
+      str->getPrefix(aOutString);
+      MOZ_ASSERT(aOutString.Length() == iter.Key());
       pick = str;
       continue;
     }
 
-    nsDependentCSubstring cur(str->get(), iter.Key());
-    if (cur < aOutString) {
-      aOutString.Rebind(str->get(), iter.Key());
+    nsAutoCString cur;
+    str->getPrefix(cur);
+    if (!cur.IsEmpty() && cur < aOutString) {
+      aOutString.Assign(cur);
+      MOZ_ASSERT(aOutString.Length() == iter.Key());
       pick = str;
     }
   }
@@ -598,5 +569,7 @@ bool VLPrefixSet::GetSmallestPrefix(nsDependentCSubstring& aOutString) {
   return pick != nullptr;
 }
 
+nsresult LookupCacheV4::LoadMozEntries() { return NS_ERROR_NOT_IMPLEMENTED; }
+
 }  // namespace safebrowsing
 }  // namespace mozilla
