@@ -2,33 +2,183 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-
-const bundle = Services.strings.createBundle(
-  "chrome://global/locale/aboutUrlClassifier.properties");
+const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 const UPDATE_BEGIN = "safebrowsing-update-begin";
 const UPDATE_FINISH = "safebrowsing-update-finished";
 const JSLOG_PREF = "browser.safebrowsing.debug";
 
-const STR_NA = bundle.GetStringFromName("NotAvailable");
-
-function unLoad() {
-  window.removeEventListener("unload", unLoad);
-
+window.onunload = function() {
+  Search.uninit();
   Provider.uninit();
   Cache.uninit();
   Debug.uninit();
-}
-
-function onLoad() {
-  window.removeEventListener("load", onLoad);
-  window.addEventListener("unload", unLoad);
+};
 
+window.onload = function() {
+  Search.init();
   Provider.init();
   Cache.init();
   Debug.init();
-}
+};
+
+/*
+ * Search
+ */
+var Search = {
+  init() {
+    let classifier = Cc["@mozilla.org/url-classifier/dbservice;1"].getService(
+      Ci.nsIURIClassifier
+    );
+    let featureNames = classifier.getFeatureNames();
+
+    let fragment = document.createDocumentFragment();
+    featureNames.forEach(featureName => {
+      let div = document.createElement("div");
+      fragment.appendChild(div);
+
+      let checkbox = document.createElement("input");
+      checkbox.id = "feature_" + featureName;
+      checkbox.type = "checkbox";
+      checkbox.checked = true;
+      div.appendChild(checkbox);
+
+      let label = document.createElement("label");
+      label.for = checkbox.id;
+      div.appendChild(label);
+
+      let text = document.createTextNode(featureName);
+      label.appendChild(text);
+    });
+
+    let list = document.getElementById("search-features");
+    list.appendChild(fragment);
+
+    let btn = document.getElementById("search-button");
+    btn.addEventListener("click", this.search);
+
+    this.hideError();
+    this.hideResults();
+  },
+
+  uninit() {
+    let list = document.getElementById("search-features");
+    while (list.firstChild) {
+      list.firstChild.remove();
+    }
+
+    let btn = document.getElementById("search-button");
+    btn.removeEventListener("click", this.search);
+  },
+
+  search() {
+    Search.hideError();
+    Search.hideResults();
+
+    let input = document.getElementById("search-input").value;
+
+    let uri;
+    try {
+      uri = Services.io.newURI(input);
+      if (!uri) {
+        Search.reportError("url-classifier-search-error-invalid-url");
+        return;
+      }
+    } catch (ex) {
+      Search.reportError("url-classifier-search-error-invalid-url");
+      return;
+    }
+
+    let classifier = Cc["@mozilla.org/url-classifier/dbservice;1"].getService(
+      Ci.nsIURIClassifier
+    );
+
+    let featureNames = classifier.getFeatureNames();
+    let features = [];
+    featureNames.forEach(featureName => {
+      if (document.getElementById("feature_" + featureName).checked) {
+        let feature = classifier.getFeatureByName(featureName);
+        if (feature) {
+          features.push(feature);
+        }
+      }
+    });
+
+    if (!features.length) {
+      Search.reportError("url-classifier-search-error-no-features");
+      return;
+    }
+
+    let listType =
+      document.getElementById("search-listtype").value == 0
+        ? Ci.nsIUrlClassifierFeature.blacklist
+        : Ci.nsIUrlClassifierFeature.whitelist;
+    classifier.asyncClassifyLocalWithFeatures(uri, features, listType, list =>
+      Search.showResults(list)
+    );
+
+    Search.hideError();
+  },
+
+  hideError() {
+    let errorMessage = document.getElementById("search-error-message");
+    errorMessage.style.display = "none";
+  },
+
+  reportError(msg) {
+    let errorMessage = document.getElementById("search-error-message");
+    document.l10n.setAttributes(errorMessage, msg);
+    errorMessage.style.display = "";
+  },
+
+  hideResults() {
+    let resultTitle = document.getElementById("result-title");
+    resultTitle.style.display = "none";
+
+    let resultTable = document.getElementById("result-table");
+    resultTable.style.display = "none";
+  },
+
+  showResults(results) {
+    let fragment = document.createDocumentFragment();
+    results.forEach(result => {
+      let tr = document.createElement("tr");
+      fragment.appendChild(tr);
+
+      let th = document.createElement("th");
+      tr.appendChild(th);
+      th.appendChild(document.createTextNode(result.feature.name));
+
+      let td = document.createElement("td");
+      tr.appendChild(td);
+
+      let featureName = document.createElement("div");
+      document.l10n.setAttributes(
+        featureName,
+        "url-classifier-search-result-uri",
+        { uri: result.uri.spec }
+      );
+      td.appendChild(featureName);
+
+      let list = document.createElement("div");
+      document.l10n.setAttributes(list, "url-classifier-search-result-list", {
+        list: result.list,
+      });
+      td.appendChild(list);
+    });
+
+    let resultTable = document.getElementById("result-table");
+    while (resultTable.firstChild) {
+      resultTable.firstChild.remove();
+    }
+
+    resultTable.appendChild(fragment);
+    resultTable.style.display = "";
+
+    let resultTitle = document.getElementById("result-title");
+    resultTitle.style.display = "";
+  },
+};
 
 /*
  * Provider
@@ -41,9 +191,12 @@ var Provider = {
   init() {
     this.providers = new Set();
     let branch = Services.prefs.getBranch("browser.safebrowsing.provider.");
-    let children = branch.getChildList("", {});
+    let children = branch.getChildList("");
     for (let child of children) {
-      this.providers.add(child.split(".")[0]);
+      let provider = child.split(".")[0];
+      if (this.isActiveProvider(provider)) {
+        this.providers.add(provider);
+      }
     }
 
     this.register();
@@ -64,7 +217,7 @@ var Provider = {
     document.getElementById("update-" + p).disabled = true;
 
     let elem = document.getElementById(p + "-col-lastupdateresult");
-    elem.childNodes[0].nodeValue = bundle.GetStringFromName("Updating");
+    document.l10n.setAttributes(elem, "url-classifier-updating");
   },
 
   onFinishUpdate(aSubject, aTopic, aData) {
@@ -84,13 +237,15 @@ var Provider = {
 
     let elem = document.getElementById(p + "-col-lastupdateresult");
     if (aData.startsWith("success")) {
-      elem.childNodes[0].nodeValue = bundle.GetStringFromName("success");
+      document.l10n.setAttributes(elem, "url-classifier-success");
     } else if (aData.startsWith("update error")) {
-      elem.childNodes[0].nodeValue =
-        bundle.formatStringFromName("updateError", [aData.split(": ")[1]], 1);
+      document.l10n.setAttributes(elem, "url-classifier-update-error", {
+        error: aData.split(": ")[1],
+      });
     } else if (aData.startsWith("download error")) {
-      elem.childNodes[0].nodeValue =
-        bundle.formatStringFromName("downloadError", [aData.split(": ")[1]], 1);
+      document.l10n.setAttributes(elem, "url-classifier-download-error", {
+        error: aData.split(": ")[1],
+      });
     } else {
       elem.childNodes[0].nodeValue = aData;
     }
@@ -124,14 +279,16 @@ var Provider = {
         if (column.id === "col-update") {
           let btn = document.createElement("button");
           btn.id = "update-" + provider;
-          btn.addEventListener("click", () => { this.update(provider); });
+          btn.addEventListener("click", () => {
+            this.update(provider);
+          });
 
-          let str = bundle.GetStringFromName("TriggerUpdate");
-          btn.appendChild(document.createTextNode(str));
+          document.l10n.setAttributes(btn, "url-classifier-trigger-update");
           td.appendChild(btn);
+        } else if (column.id === "col-lastupdateresult") {
+          document.l10n.setAttributes(td, "url-classifier-not-available");
         } else {
-          let str = column.id === "col-lastupdateresult" ? STR_NA : "";
-          td.appendChild(document.createTextNode(str));
+          td.appendChild(document.createTextNode(""));
         }
         tr.appendChild(td);
       }
@@ -144,30 +301,38 @@ var Provider = {
       let values = {};
       values["col-provider"] = provider;
 
-      let pref = "browser.safebrowsing.provider." + provider + ".lastupdatetime";
+      let pref =
+        "browser.safebrowsing.provider." + provider + ".lastupdatetime";
       let lut = Services.prefs.getCharPref(pref, "");
-      values["col-lastupdatetime"] = lut ? new Date(lut * 1) : STR_NA;
+      values["col-lastupdatetime"] = lut ? new Date(lut * 1) : null;
 
       pref = "browser.safebrowsing.provider." + provider + ".nextupdatetime";
       let nut = Services.prefs.getCharPref(pref, "");
-      values["col-nextupdatetime"] = nut ? new Date(nut * 1) : STR_NA;
+      values["col-nextupdatetime"] = nut ? new Date(nut * 1) : null;
 
-      let listmanager = Cc["@mozilla.org/url-classifier/listmanager;1"]
-                        .getService(Ci.nsIUrlListManager);
+      let listmanager = Cc[
+        "@mozilla.org/url-classifier/listmanager;1"
+      ].getService(Ci.nsIUrlListManager);
       let bot = listmanager.getBackOffTime(provider);
-      values["col-backofftime"] = bot ? new Date(bot * 1) : STR_NA;
+      values["col-backofftime"] = bot ? new Date(bot * 1) : null;
 
       for (let key of Object.keys(values)) {
         let elem = document.getElementById(provider + "-" + key);
-        elem.childNodes[0].nodeValue = values[key];
+        if (values[key]) {
+          elem.removeAttribute("data-l10n-id");
+          elem.childNodes[0].nodeValue = values[key];
+        } else {
+          document.l10n.setAttributes(elem, "url-classifier-not-available");
+        }
       }
     }
   },
 
   // Call update for the provider.
   update(provider) {
-    let listmanager = Cc["@mozilla.org/url-classifier/listmanager;1"]
-                      .getService(Ci.nsIUrlListManager);
+    let listmanager = Cc[
+      "@mozilla.org/url-classifier/listmanager;1"
+    ].getService(Ci.nsIUrlListManager);
 
     let pref = "browser.safebrowsing.provider." + provider + ".lists";
     let tables = Services.prefs.getCharPref(pref, "");
@@ -175,10 +340,30 @@ var Provider = {
     if (!listmanager.forceUpdates(tables)) {
       // This may because of back-off algorithm.
       let elem = document.getElementById(provider + "-col-lastupdateresult");
-      elem.childNodes[0].nodeValue = bundle.GetStringFromName("CannotUpdate");
+      document.l10n.setAttributes(elem, "url-classifier-cannot-update");
     }
   },
 
+  // if we can find any table registered an updateURL in the listmanager,
+  // the provider is active. This is used to filter out google v2 provider
+  // without changing the preference.
+  isActiveProvider(provider) {
+    let listmanager = Cc[
+      "@mozilla.org/url-classifier/listmanager;1"
+    ].getService(Ci.nsIUrlListManager);
+
+    let pref = "browser.safebrowsing.provider." + provider + ".lists";
+    let tables = Services.prefs.getCharPref(pref, "").split(",");
+
+    for (let i = 0; i < tables.length; i++) {
+      let updateUrl = listmanager.getUpdateUrl(tables[i]);
+      if (updateUrl) {
+        return true;
+      }
+    }
+
+    return false;
+  },
 };
 
 /*
@@ -208,16 +393,21 @@ var Cache = {
     this.createCacheEntries();
 
     let refreshBtn = document.getElementById("refresh-cache-btn");
-    refreshBtn.addEventListener("click", () => { this.refresh(); });
+    refreshBtn.addEventListener("click", () => {
+      this.refresh();
+    });
 
     let clearBtn = document.getElementById("clear-cache-btn");
     clearBtn.addEventListener("click", () => {
-      let dbservice = Cc["@mozilla.org/url-classifier/dbservice;1"]
-                      .getService(Ci.nsIUrlClassifierDBService);
+      let dbservice = Cc["@mozilla.org/url-classifier/dbservice;1"].getService(
+        Ci.nsIUrlClassifierDBService
+      );
       dbservice.clearCache();
       // Since clearCache is async call, we just simply assume it will be
       // updated in 100 milli-seconds.
-      setTimeout(() => { this.refresh(); }, 100);
+      setTimeout(() => {
+        this.refresh();
+      }, 100);
     });
   },
 
@@ -246,15 +436,27 @@ var Cache = {
         if (i == 0 && tds.length != cols) {
           td.setAttribute("colspan", cols - tds.length + 1);
         }
-        let elem = typeof v === "object" ? v : document.createTextNode(v);
-        td.appendChild(elem);
+
+        if (typeof v === "object") {
+          if (v.l10n) {
+            document.l10n.setAttributes(td, v.l10n);
+          } else {
+            td.removeAttribute("data-l10n-id");
+            td.appendChild(v);
+          }
+        } else {
+          td.removeAttribute("data-l10n-id");
+          td.textContent = v;
+        }
+
         tr.appendChild(td);
       });
       body.appendChild(tr);
     }
 
-    let dbservice = Cc["@mozilla.org/url-classifier/dbservice;1"]
-                    .getService(Ci.nsIUrlClassifierInfo);
+    let dbservice = Cc["@mozilla.org/url-classifier/dbservice;1"].getService(
+      Ci.nsIUrlClassifierInfo
+    );
 
     for (let provider of Provider.providers) {
       let pref = "browser.safebrowsing.provider." + provider + ".lists";
@@ -262,7 +464,7 @@ var Cache = {
 
       for (let table of tables) {
         dbservice.getCacheInfo(table, {
-          onGetCacheComplete: (aCache) => {
+          onGetCacheComplete: aCache => {
             let entries = aCache.entries;
             if (entries.length === 0) {
               this.showCacheEnties.delete(table);
@@ -270,8 +472,11 @@ var Cache = {
             }
 
             let positiveCacheCount = 0;
-            for (let i = 0; i < entries.length ; i++) {
-              let entry = entries.queryElementAt(i, Ci.nsIUrlClassifierCacheEntry);
+            for (let i = 0; i < entries.length; i++) {
+              let entry = entries.queryElementAt(
+                i,
+                Ci.nsIUrlClassifierCacheEntry
+              );
               let matches = entry.matches;
               positiveCacheCount += matches.length;
 
@@ -281,18 +486,34 @@ var Cache = {
                 continue;
               }
 
-              let tds = [table, entry.prefix, new Date(entry.expiry * 1000).toString()];
+              let tds = [
+                table,
+                entry.prefix,
+                new Date(entry.expiry * 1000).toString(),
+              ];
               let j = 0;
               do {
                 if (matches.length >= 1) {
-                  let match =
-                    matches.queryElementAt(j, Ci.nsIUrlClassifierPositiveCacheEntry);
-                  let list = [match.fullhash, new Date(match.expiry * 1000).toString()];
+                  let match = matches.queryElementAt(
+                    j,
+                    Ci.nsIUrlClassifierPositiveCacheEntry
+                  );
+                  let list = [
+                    match.fullhash,
+                    new Date(match.expiry * 1000).toString(),
+                  ];
                   tds = tds.concat(list);
                 } else {
-                  tds = tds.concat([STR_NA, STR_NA]);
+                  tds = tds.concat([
+                    { l10n: "url-classifier-not-available" },
+                    { l10n: "url-classifier-not-available" },
+                  ]);
                 }
-                createRow(tds, document.getElementById("cache-entries-table-body"), 5);
+                createRow(
+                  tds,
+                  document.getElementById("cache-entries-table-body"),
+                  5
+                );
                 j++;
                 tds = [""];
               } while (j < matches.length);
@@ -312,14 +533,19 @@ var Cache = {
             });
 
             let tds = [table, entries.length, positiveCacheCount, chk];
-            createRow(tds, document.getElementById("cache-table-body"), tds.length);
-          }
+            createRow(
+              tds,
+              document.getElementById("cache-table-body"),
+              tds.length
+            );
+          },
         });
       }
     }
 
     let entries_div = document.getElementById("cache-entries");
-    entries_div.style.display = this.showCacheEnties.size == 0 ? "none" : "block";
+    entries_div.style.display =
+      this.showCacheEnties.size == 0 ? "none" : "block";
   },
 };
 
@@ -328,12 +554,15 @@ var Cache = {
  */
 var Debug = {
   // url-classifier NSPR Log modules.
-  modules: ["UrlClassifierDbService",
-            "nsChannelClassifier",
-            "UrlClassifierProtocolParser",
-            "UrlClassifierStreamUpdater",
-            "UrlClassifierPrefixSet",
-            "ApplicationReputation"],
+  modules: [
+    "UrlClassifierDbService",
+    "nsChannelClassifier",
+    "UrlClassifier",
+    "UrlClassifierProtocolParser",
+    "UrlClassifierStreamUpdater",
+    "UrlClassifierPrefixSet",
+    "ApplicationReputation",
+  ],
 
   init() {
     this.register();
@@ -387,7 +616,9 @@ var Debug = {
       chk.id = "chk-" + module;
       chk.type = "checkbox";
       chk.checked = true;
-      chk.addEventListener("click", () => { logModuleUpdate(module); });
+      chk.addEventListener("click", () => {
+        logModuleUpdate(module);
+      });
       container.appendChild(chk, modules);
 
       let label = document.createElement("label");
@@ -416,14 +647,16 @@ var Debug = {
 
     // Disable configure log modules if log modules are already set
     // by environment variable.
-    let env = Cc["@mozilla.org/process/environment;1"]
-              .getService(Ci.nsIEnvironment);
+    let env = Cc["@mozilla.org/process/environment;1"].getService(
+      Ci.nsIEnvironment
+    );
 
-    let logModules = env.get("MOZ_LOG") ||
-                     env.get("MOZ_LOG_MODULES") ||
-                     env.get("NSPR_LOG_MODULES");
+    let logModules =
+      env.get("MOZ_LOG") ||
+      env.get("MOZ_LOG_MODULES") ||
+      env.get("NSPR_LOG_MODULES");
 
-    if (logModules.length > 0) {
+    if (logModules.length) {
       document.getElementById("set-log-modules").disabled = true;
       for (let module of this.modules) {
         document.getElementById("chk-" + module).disabled = true;
@@ -436,7 +669,7 @@ var Debug = {
     // Disable set log file if log file is already set
     // by environment variable.
     let logFile = env.get("MOZ_LOG_FILE") || env.get("NSPR_LOG_FILE");
-    if (logFile.length > 0) {
+    if (logFile.length) {
       document.getElementById("set-log-file").disabled = true;
       document.getElementById("log-file").value = logFile;
     }
@@ -449,9 +682,11 @@ var Debug = {
     jsChk.checked = enabled;
 
     let curJSLog = document.getElementById("cur-js-log");
-    curJSLog.childNodes[0].nodeValue = enabled ?
-      bundle.GetStringFromName("Enabled") :
-      bundle.GetStringFromName("Disabled");
+    if (enabled) {
+      document.l10n.setAttributes(curJSLog, "url-classifier-enabled");
+    } else {
+      document.l10n.setAttributes(curJSLog, "url-classifier-disabled");
+    }
   },
 
   jslog() {
@@ -461,7 +696,7 @@ var Debug = {
 
   nsprlog() {
     // Turn off debugging for all the modules.
-    let children = Services.prefs.getBranch("logging.").getChildList("", {});
+    let children = Services.prefs.getBranch("logging.").getChildList("");
     for (let pref of children) {
       if (!pref.startsWith("config.")) {
         Services.prefs.clearUserPref(`logging.${pref}`);
@@ -485,5 +720,5 @@ var Debug = {
 
     let curLogFile = document.getElementById("cur-log-file");
     curLogFile.childNodes[0].nodeValue = logFile;
-  }
+  },
 };
