@@ -6,9 +6,12 @@
 // that aren't initialized outside of a XUL app environment like AddonManager
 // and the "@mozilla.org/xre/app-info;1" component.
 
-ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource://gre/modules/Troubleshoot.jsm");
+const { AppConstants } = ChromeUtils.import(
+  "resource://gre/modules/AppConstants.jsm"
+);
+const { Troubleshoot } = ChromeUtils.import(
+  "resource://gre/modules/Troubleshoot.jsm"
+);
 
 function test() {
   waitForExplicitFinish();
@@ -29,7 +32,6 @@ registerCleanupFunction(function() {
 });
 
 var tests = [
-
   function snapshotSchema(done) {
     Troubleshoot.snapshot(function(snapshot) {
       try {
@@ -55,15 +57,24 @@ var tests = [
     });
     Troubleshoot.snapshot(function(snapshot) {
       let p = snapshot.modifiedPreferences;
-      is(p["javascript.troubleshoot"], true,
-         "The pref should be present because it's whitelisted " +
-         "but not blacklisted.");
-      ok(!("troubleshoot.foo" in p),
-         "The pref should be absent because it's not in the whitelist.");
-      ok(!("javascript.print_to_filename" in p),
-         "The pref should be absent because it's blacklisted.");
-      ok(!("network.proxy.troubleshoot" in p),
-         "The pref should be absent because it's blacklisted.");
+      is(
+        p["javascript.troubleshoot"],
+        true,
+        "The pref should be present because it's whitelisted " +
+          "but not blacklisted."
+      );
+      ok(
+        !("troubleshoot.foo" in p),
+        "The pref should be absent because it's not in the whitelist."
+      );
+      ok(
+        !("javascript.print_to_filename" in p),
+        "The pref should be absent because it's blacklisted."
+      );
+      ok(
+        !("network.proxy.troubleshoot" in p),
+        "The pref should be absent because it's blacklisted."
+      );
       prefs.forEach(p => Services.prefs.deleteBranch(p));
       done();
     });
@@ -83,7 +94,7 @@ var tests = [
       Services.prefs.deleteBranch(name);
       done();
     });
-  }
+  },
 ];
 
 // This is inspired by JSON Schema, or by the example on its Wikipedia page
@@ -108,6 +119,10 @@ const SNAPSHOT_SCHEMA = {
           required: true,
           type: "string",
         },
+        distributionID: {
+          required: true,
+          type: "string",
+        },
         userAgent: {
           required: true,
           type: "string",
@@ -125,6 +140,9 @@ const SNAPSHOT_SCHEMA = {
         supportURL: {
           type: "string",
         },
+        launcherProcessState: {
+          type: "number",
+        },
         remoteAutoStart: {
           type: "boolean",
           required: true,
@@ -138,27 +156,6 @@ const SNAPSHOT_SCHEMA = {
         numRemoteWindows: {
           type: "number",
         },
-        currentContentProcesses: {
-          type: "number",
-        },
-        maxContentProcesses: {
-          type: "number",
-        },
-        styloBuild: {
-          type: "boolean",
-        },
-        styloDefault: {
-          type: "boolean",
-        },
-        styloResult: {
-          type: "boolean",
-        },
-        styloChromeDefault: {
-          type: "boolean",
-        },
-        styloChromeResult: {
-          type: "boolean",
-        },
         policiesStatus: {
           type: "number",
         },
@@ -271,6 +268,20 @@ const SNAPSHOT_SCHEMA = {
         },
       },
     },
+    processes: {
+      required: true,
+      type: "object",
+      properties: {
+        maxWebContentProcesses: {
+          required: true,
+          type: "number",
+        },
+        remoteTypes: {
+          required: true,
+          type: "object",
+        },
+      },
+    },
     modifiedPreferences: {
       required: true,
       type: "object",
@@ -278,6 +289,16 @@ const SNAPSHOT_SCHEMA = {
     lockedPreferences: {
       required: true,
       type: "object",
+      properties: {
+        "fission.autostart": {
+          required: false,
+          type: "boolean",
+        },
+        "dom.ipc.processCount.webIsolated": {
+          required: false,
+          type: "number",
+        },
+      },
     },
     graphics: {
       required: true,
@@ -301,7 +322,17 @@ const SNAPSHOT_SCHEMA = {
           type: "boolean",
         },
         numAcceleratedWindowsMessage: {
-          type: "array",
+          type: "object",
+          properties: {
+            key: {
+              required: true,
+              type: "string",
+            },
+            args: {
+              required: false,
+              type: "object",
+            },
+          },
         },
         adapterDescription: {
           type: "string",
@@ -316,11 +347,14 @@ const SNAPSHOT_SCHEMA = {
           type: "string",
         },
         adapterRAM: {
-          type: "string",
+          type: "number",
         },
         adapterDrivers: {
           type: "string",
         },
+        driverVendor: {
+          type: "string",
+        },
         driverVersion: {
           type: "string",
         },
@@ -340,11 +374,14 @@ const SNAPSHOT_SCHEMA = {
           type: "string",
         },
         adapterRAM2: {
-          type: "string",
+          type: "number",
         },
         adapterDrivers2: {
           type: "string",
         },
+        driverVendor2: {
+          type: "string",
+        },
         driverVersion2: {
           type: "string",
         },
@@ -366,6 +403,9 @@ const SNAPSHOT_SCHEMA = {
         usesTiling: {
           type: "boolean",
         },
+        contentUsesTiling: {
+          type: "boolean",
+        },
         offMainThreadPaintEnabled: {
           type: "boolean",
         },
@@ -409,9 +449,16 @@ const SNAPSHOT_SCHEMA = {
           type: "object",
         },
         failures: {
-          type: "array",
-          items: {
-            type: "string",
+          type: "object",
+          properties: {
+            key: {
+              required: true,
+              type: "string",
+            },
+            args: {
+              required: false,
+              type: "object",
+            },
           },
         },
         indices: {
@@ -427,7 +474,26 @@ const SNAPSHOT_SCHEMA = {
           type: "array",
         },
         direct2DEnabledMessage: {
-          type: "array",
+          type: "object",
+          properties: {
+            key: {
+              required: true,
+              type: "string",
+            },
+            args: {
+              required: false,
+              type: "object",
+            },
+          },
+        },
+        targetFrameRate: {
+          type: "number",
+        },
+        windowProtocol: {
+          type: "string",
+        },
+        desktopEnvironment: {
+          type: "string",
         },
       },
     },
@@ -443,10 +509,6 @@ const SNAPSHOT_SCHEMA = {
           required: true,
           type: "number",
         },
-        currentPreferredChannelLayout: {
-          required: true,
-          type: "string",
-        },
         currentPreferredSampleRate: {
           required: true,
           type: "number",
@@ -512,7 +574,7 @@ const SNAPSHOT_SCHEMA = {
               minLatency: {
                 required: true,
                 type: "number",
-              }
+              },
             },
           },
         },
@@ -577,21 +639,12 @@ const SNAPSHOT_SCHEMA = {
               minLatency: {
                 required: true,
                 type: "number",
-              }
+              },
             },
           },
         },
       },
     },
-    javaScript: {
-      required: true,
-      type: "object",
-      properties: {
-        incrementalGCEnabled: {
-          type: "boolean",
-        },
-      },
-    },
     accessibility: {
       required: true,
       type: "object",
@@ -607,8 +660,8 @@ const SNAPSHOT_SCHEMA = {
           type: "boolean",
         },
         instantiator: {
-          type: "string"
-        }
+          type: "string",
+        },
       },
     },
     libraryVersions: {
@@ -697,44 +750,41 @@ const SNAPSHOT_SCHEMA = {
         },
       },
     },
-    experiments: {
-      type: "array",
-    },
     sandbox: {
       required: false,
       type: "object",
       properties: {
         hasSeccompBPF: {
           required: AppConstants.platform == "linux",
-          type: "boolean"
+          type: "boolean",
         },
         hasSeccompTSync: {
           required: AppConstants.platform == "linux",
-          type: "boolean"
+          type: "boolean",
         },
         hasUserNamespaces: {
           required: AppConstants.platform == "linux",
-          type: "boolean"
+          type: "boolean",
         },
         hasPrivilegedUserNamespaces: {
           required: AppConstants.platform == "linux",
-          type: "boolean"
+          type: "boolean",
         },
         canSandboxContent: {
           required: false,
-          type: "boolean"
+          type: "boolean",
         },
         canSandboxMedia: {
           required: false,
-          type: "boolean"
+          type: "boolean",
         },
         contentSandboxLevel: {
-          required: AppConstants.MOZ_CONTENT_SANDBOX,
-          type: "number"
+          required: AppConstants.MOZ_SANDBOX,
+          type: "number",
         },
         effectiveContentSandboxLevel: {
-          required: AppConstants.MOZ_CONTENT_SANDBOX,
-          type: "number"
+          required: AppConstants.MOZ_SANDBOX,
+          type: "number",
         },
         syscallLog: {
           required: AppConstants.platform == "linux",
@@ -774,6 +824,28 @@ const SNAPSHOT_SCHEMA = {
         },
       },
     },
+    startupCache: {
+      required: false,
+      type: "object",
+      properties: {
+        DiskCachePath: {
+          required: true,
+          type: "string",
+        },
+        IgnoreDiskCache: {
+          required: true,
+          type: "boolean",
+        },
+        FoundDiskCacheOnInit: {
+          required: true,
+          type: "boolean",
+        },
+        WroteToDiskCache: {
+          required: true,
+          type: "boolean",
+        },
+      },
+    },
     intl: {
       required: true,
       type: "object",
@@ -784,23 +856,23 @@ const SNAPSHOT_SCHEMA = {
           properties: {
             requested: {
               required: true,
-              type: "array"
+              type: "array",
             },
             available: {
               required: true,
-              type: "array"
+              type: "array",
             },
             supported: {
               required: true,
-              type: "array"
+              type: "array",
             },
             regionalPrefs: {
               required: true,
-              type: "array"
+              type: "array",
             },
             defaultLocale: {
               required: true,
-              type: "string"
+              type: "string",
             },
           },
         },
@@ -810,16 +882,29 @@ const SNAPSHOT_SCHEMA = {
           properties: {
             systemLocales: {
               required: true,
-              type: "array"
+              type: "array",
             },
             regionalPrefsLocales: {
               required: true,
-              type: "array"
+              type: "array",
             },
           },
         },
       },
     },
+    remoteAgent: {
+      type: "object",
+      properties: {
+        listening: {
+          required: true,
+          type: "boolean",
+        },
+        url: {
+          required: true,
+          type: "string",
+        },
+      },
+    },
   },
 };
 
@@ -833,35 +918,48 @@ const SNAPSHOT_SCHEMA = {
  * @param schema The schema that obj should conform to.
  */
 function validateObject(obj, schema) {
-  if (obj === undefined && !schema.required)
+  if (obj === undefined && !schema.required) {
     return;
-  if (typeof(schema.type) != "string")
+  }
+  if (typeof schema.type != "string") {
     throw schemaErr("'type' must be a string", schema);
-  if (objType(obj) != schema.type)
+  }
+  if (objType(obj) != schema.type) {
     throw validationErr("Object is not of the expected type", obj, schema);
+  }
   let validatorFnName = "validateObject_" + schema.type;
-  if (!(validatorFnName in this))
+  if (!(validatorFnName in this)) {
     throw schemaErr("Validator function not defined for type", schema);
+  }
   this[validatorFnName](obj, schema);
 }
 
 function validateObject_object(obj, schema) {
-  if (typeof(schema.properties) != "object")
+  if (typeof schema.properties != "object") {
     // Don't care what obj's properties are.
     return;
+  }
   // First check that all the schema's properties match the object.
-  for (let prop in schema.properties)
+  for (let prop in schema.properties) {
     validateObject(obj[prop], schema.properties[prop]);
+  }
   // Now check that the object doesn't have any properties not in the schema.
-  for (let prop in obj)
-    if (!(prop in schema.properties))
-      throw validationErr("Object has property " + prop + " not in schema", obj, schema);
+  for (let prop in obj) {
+    if (!(prop in schema.properties)) {
+      throw validationErr(
+        "Object has property " + prop + " not in schema",
+        obj,
+        schema
+      );
+    }
+  }
 }
 
 function validateObject_array(array, schema) {
-  if (typeof(schema.items) != "object")
+  if (typeof schema.items != "object") {
     // Don't care what the array's elements are.
     return;
+  }
   array.forEach(elt => validateObject(elt, schema.items));
 }
 
@@ -870,9 +968,14 @@ function validateObject_boolean(bool, schema) {}
 function validateObject_number(num, schema) {}
 
 function validationErr(msg, obj, schema) {
-  return new Error("Validation error: " + msg +
-                   ": object=" + JSON.stringify(obj) +
-                   ", schema=" + JSON.stringify(schema));
+  return new Error(
+    "Validation error: " +
+      msg +
+      ": object=" +
+      JSON.stringify(obj) +
+      ", schema=" +
+      JSON.stringify(schema)
+  );
 }
 
 function schemaErr(msg, schema) {
@@ -880,12 +983,15 @@ function schemaErr(msg, schema) {
 }
 
 function objType(obj) {
-  let type = typeof(obj);
-  if (type != "object")
+  let type = typeof obj;
+  if (type != "object") {
     return type;
-  if (Array.isArray(obj))
+  }
+  if (Array.isArray(obj)) {
     return "array";
-  if (obj === null)
+  }
+  if (obj === null) {
     return "null";
+  }
   return type;
 }
