@@ -15,8 +15,11 @@ function test() {
   delete Services.ww;
   is(Services.ww, undefined, "Services.ww should now be undefined");
 
-  var pm = Services.perms;
-  pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+  PermissionTestUtils.add(
+    "http://example.com/",
+    "install",
+    Services.perms.ALLOW_ACTION
+  );
 
   var triggers = encodeURIComponent(
     JSON.stringify({
@@ -51,7 +54,7 @@ function finish_test(count) {
 
   Services.ww = gWindowWatcher;
 
-  Services.perms.remove(makeURI("http://example.com"), "install");
+  PermissionTestUtils.remove("http://example.com", "install");
 
   Harness.finish();
 }
