@@ -19,24 +19,24 @@ typedef char NS_tchar;
 // This class provides an API to extract files from an update archive.
 class ArchiveReader {
  public:
-  ArchiveReader() {}
+  ArchiveReader() = default;
   ~ArchiveReader() { Close(); }
 
-  int Open(const NS_tchar *path);
+  int Open(const NS_tchar* path);
   int VerifySignature();
-  int VerifyProductInformation(const char *MARChannelID,
-                               const char *appVersion);
+  int VerifyProductInformation(const char* MARChannelID,
+                               const char* appVersion);
   void Close();
 
-  int ExtractFile(const char *item, const NS_tchar *destination);
-  int ExtractFileToStream(const char *item, FILE *fp);
+  int ExtractFile(const char* item, const NS_tchar* destination);
+  int ExtractFileToStream(const char* item, FILE* fp);
 
  private:
-  int ExtractItemToStream(const MarItem *item, FILE *fp);
+  int ExtractItemToStream(const MarItem* item, FILE* fp);
 
-  MarFile *mArchive = nullptr;
-  uint8_t *mInBuf = nullptr;
-  uint8_t *mOutBuf = nullptr;
+  MarFile* mArchive = nullptr;
+  uint8_t* mInBuf = nullptr;
+  uint8_t* mOutBuf = nullptr;
   size_t mInBufSize = 262144;
   size_t mOutBufSize = 262144;
 };
