@@ -32,7 +32,7 @@ class PuppetWidget;
  */
 
 class TextEventDispatcher final {
-  ~TextEventDispatcher() {}
+  ~TextEventDispatcher() = default;
 
   NS_INLINE_DECL_REFCOUNTING(TextEventDispatcher)
 
@@ -84,6 +84,18 @@ class TextEventDispatcher final {
     return mIMENotificationRequests;
   }
 
+  /**
+   * OnWidgetChangeIMENotificationRequests() is called when aWidget's
+   * IMENotificationRequest is maybe modified by unusual path.  E.g.,
+   * modified in an async path.
+   */
+  void OnWidgetChangeIMENotificationRequests(nsIWidget* aWidget) {
+    MOZ_ASSERT(aWidget);
+    if (mWidget == aWidget) {
+      UpdateNotificationRequests();
+    }
+  }
+
   /**
    * GetState() returns current state of this class.
    *
@@ -381,12 +393,10 @@ class TextEventDispatcher final {
     // tests because this instance won't dispatch the events via the parent
     // process again.
     eSameProcessSyncTestInputTransaction,
-    // Input transaction for Others (must be IME on B2G).  Events are fired
-    // synchronously because TextInputProcessor which is the only user of
-    // this input transaction type supports only keyboard apps on B2G.
-    // Keyboard apps on B2G doesn't want to dispatch keyboard events to
-    // chrome process. Therefore, this should dispatch key events only in
-    // the current process.
+    // Input transaction for others (currently, only FuzzingFunctions).
+    // Events are fired synchronously in the process.
+    // XXX Should we make this async for testing default action handlers in
+    //     the main process?
     eSameProcessSyncInputTransaction
   };
 
@@ -430,13 +440,6 @@ class TextEventDispatcher final {
   // received yet.  Otherwise, false.
   bool mHasFocus;
 
-  // If this is true, keydown and keyup events are dispatched even when there
-  // is a composition.
-  static bool sDispatchKeyEventsDuringComposition;
-  // If this is true, keypress events for non-printable keys are dispatched only
-  // for event listeners of the system event group in web content.
-  static bool sDispatchKeyPressEventsOnlySystemGroupInContent;
-
   nsresult BeginInputTransactionInternal(TextEventDispatcherListener* aListener,
                                          InputTransactionType aType);
 
