@@ -4,9 +4,9 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "base/message_loop.h"
-#include "GeneratedJNIWrappers.h"
 #include "mozilla/Atomics.h"
 #include "mozilla/EventQueue.h"
+#include "mozilla/java/GeckoThreadWrappers.h"
 #include "mozilla/LinkedList.h"
 #include "mozilla/Monitor.h"
 #include "mozilla/Mutex.h"
@@ -70,9 +70,9 @@ NS_IMETHODIMP
 AndroidUiThread::Dispatch(already_AddRefed<nsIRunnable> aEvent,
                           uint32_t aFlags) {
   if (aFlags & NS_DISPATCH_SYNC) {
-    return nsThread::Dispatch(Move(aEvent), aFlags);
+    return nsThread::Dispatch(std::move(aEvent), aFlags);
   } else {
-    EnqueueTask(Move(aEvent), 0);
+    EnqueueTask(std::move(aEvent), 0);
     return NS_OK;
   }
 }
@@ -80,7 +80,7 @@ AndroidUiThread::Dispatch(already_AddRefed<nsIRunnable> aEvent,
 NS_IMETHODIMP
 AndroidUiThread::DelayedDispatch(already_AddRefed<nsIRunnable> aEvent,
                                  uint32_t aDelayMs) {
-  EnqueueTask(Move(aEvent), aDelayMs);
+  EnqueueTask(std::move(aEvent), aDelayMs);
   return NS_OK;
 }
 
@@ -121,7 +121,7 @@ class AndroidUiTask : public LinkedListElement<AndroidUiTask> {
   using TimeDuration = mozilla::TimeDuration;
 
  public:
-  AndroidUiTask(already_AddRefed<nsIRunnable> aTask)
+  explicit AndroidUiTask(already_AddRefed<nsIRunnable> aTask)
       : mTask(aTask),
         mRunTime()  // Null timestamp representing no delay.
   {}
@@ -220,8 +220,8 @@ void EnqueueTask(already_AddRefed<nsIRunnable> aTask, int aDelayMs) {
   // add the new task into the sTaskQueue, sorted with
   // the earliest task first in the queue
   AndroidUiTask* newTask =
-      (aDelayMs ? new AndroidUiTask(mozilla::Move(aTask), aDelayMs)
-                : new AndroidUiTask(mozilla::Move(aTask)));
+      (aDelayMs ? new AndroidUiTask(std::move(aTask), aDelayMs)
+                : new AndroidUiTask(std::move(aTask)));
 
   bool headOfList = false;
   {
@@ -249,7 +249,7 @@ void EnqueueTask(already_AddRefed<nsIRunnable> aTask, int aDelayMs) {
     // if we're inserting it at the head of the queue, notify Java because
     // we need to get a callback at an earlier time than the last scheduled
     // callback
-    GeckoThread::RequestUiThreadCallback(int64_t(aDelayMs));
+    java::GeckoThread::RequestUiThreadCallback(int64_t(aDelayMs));
   }
 }
 
