@@ -15,11 +15,12 @@
 #include "WinUtils.h"
 #include "nsGkAtoms.h"
 #include "nsIDOMWindowUtils.h"
-#include "nsIDOMWheelEvent.h"
 
 #include "mozilla/MiscEvents.h"
 #include "mozilla/MouseEvents.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/dom/WheelEventBinding.h"
+#include "mozilla/StaticPrefs_mousewheel.h"
 
 #include <psapi.h>
 
@@ -278,7 +279,8 @@ nsresult MouseScrollHandler::SynthesizeNativeMouseScrollEvent(
   memset(kbdState, 0, sizeof(kbdState));
 
   AutoTArray<KeyPair, 10> keySequence;
-  WinUtils::SetupKeyModifiersSequence(&keySequence, aModifierFlags);
+  WinUtils::SetupKeyModifiersSequence(&keySequence, aModifierFlags,
+                                      aNativeMessage);
 
   for (uint32_t i = 0; i < keySequence.Length(); ++i) {
     uint8_t key = keySequence[i].mGeneral;
@@ -302,21 +304,28 @@ nsresult MouseScrollHandler::SynthesizeNativeMouseScrollEvent(
 
 /* static */
 void MouseScrollHandler::InitEvent(nsWindowBase* aWidget,
-                                   WidgetGUIEvent& aEvent,
-                                   LayoutDeviceIntPoint* aPoint) {
+                                   WidgetGUIEvent& aEvent, LPARAM* aPoint) {
   NS_ENSURE_TRUE_VOID(aWidget);
-  LayoutDeviceIntPoint point;
-  if (aPoint) {
-    point = *aPoint;
+
+  // If a point is provided, use it; otherwise, get current message point or
+  // synthetic point
+  POINTS pointOnScreen;
+  if (aPoint != nullptr) {
+    pointOnScreen = MAKEPOINTS(*aPoint);
   } else {
-    POINTS pts = GetCurrentMessagePos();
-    POINT pt;
-    pt.x = pts.x;
-    pt.y = pts.y;
-    ::ScreenToClient(aWidget->GetWindowHandle(), &pt);
-    point.x = pt.x;
-    point.y = pt.y;
+    pointOnScreen = GetCurrentMessagePos();
   }
+
+  // InitEvent expects the point to be in window coordinates, so translate the
+  // point from screen coordinates.
+  POINT pointOnWindow;
+  POINTSTOPOINT(pointOnWindow, pointOnScreen);
+  ::ScreenToClient(aWidget->GetWindowHandle(), &pointOnWindow);
+
+  LayoutDeviceIntPoint point;
+  point.x = pointOnWindow.x;
+  point.y = pointOnWindow.y;
+
   aWidget->InitEvent(aEvent, &point);
 }
 
@@ -328,6 +337,9 @@ ModifierKeyState MouseScrollHandler::GetModifierKeyState(UINT aMessage) {
   // MouseScrollHandler::Device::Elantech::HandleKeyMessage().)
   if ((aMessage == MOZ_WM_MOUSEVWHEEL || aMessage == WM_MOUSEWHEEL) &&
       !result.IsControl() && Device::Elantech::IsZooming()) {
+    // XXX Do we need to unset MODIFIER_SHIFT, MODIFIER_ALT, MODIFIER_OS too?
+    //     If one of them are true, the default action becomes not zooming.
+    result.Unset(MODIFIER_ALTGRAPH);
     result.Set(MODIFIER_CONTROL);
   }
   return result;
@@ -618,7 +630,8 @@ void MouseScrollHandler::HandleMouseWheelMessage(nsWindowBase* aWidget,
   RefPtr<nsWindowBase> kungFuDethGrip(aWidget);
 
   WidgetWheelEvent wheelEvent(true, eWheel, aWidget);
-  if (mLastEventInfo.InitWheelEvent(aWidget, wheelEvent, modKeyState)) {
+  if (mLastEventInfo.InitWheelEvent(aWidget, wheelEvent, modKeyState,
+                                    aLParam)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::HandleMouseWheelMessage: dispatching "
              "eWheel event"));
@@ -662,24 +675,26 @@ void MouseScrollHandler::HandleScrollMessageAsMouseWheelMessage(
       delta = -1.0;
       lineOrPageDelta = -1;
     case SB_PAGEDOWN:
-      wheelEvent.mDeltaMode = nsIDOMWheelEvent::DOM_DELTA_PAGE;
+      wheelEvent.mDeltaMode = dom::WheelEvent_Binding::DOM_DELTA_PAGE;
       break;
 
     case SB_LINEUP:
       delta = -1.0;
       lineOrPageDelta = -1;
     case SB_LINEDOWN:
-      wheelEvent.mDeltaMode = nsIDOMWheelEvent::DOM_DELTA_LINE;
+      wheelEvent.mDeltaMode = dom::WheelEvent_Binding::DOM_DELTA_LINE;
       break;
 
     default:
       return;
   }
   modKeyState.InitInputEvent(wheelEvent);
-  // XXX Current mouse position may not be same as when the original message
-  //     is received.  We need to know the actual mouse cursor position when
-  //     the original message was received.
-  InitEvent(aWidget, wheelEvent);
+
+  // Current mouse position may not be same as when the original message
+  // is received. However, this data is not available with the original
+  // message, which is why nullptr is passed in. We need to know the actual
+  // mouse cursor position when the original message was received.
+  InitEvent(aWidget, wheelEvent, nullptr);
 
   MOZ_LOG(
       gMouseScrollLog, LogLevel::Info,
@@ -785,13 +800,14 @@ int32_t MouseScrollHandler::LastEventInfo::RoundDelta(double aDelta) {
 
 bool MouseScrollHandler::LastEventInfo::InitWheelEvent(
     nsWindowBase* aWidget, WidgetWheelEvent& aWheelEvent,
-    const ModifierKeyState& aModKeyState) {
+    const ModifierKeyState& aModKeyState, LPARAM aLParam) {
   MOZ_ASSERT(aWheelEvent.mMessage == eWheel);
 
-  // XXX Why don't we use lParam value? We should use lParam value because
-  //     our internal message is always posted by original message handler.
-  //     So, GetMessagePos() may return different cursor position.
-  InitEvent(aWidget, aWheelEvent);
+  if (StaticPrefs::mousewheel_ignore_cursor_position_in_lparam()) {
+    InitEvent(aWidget, aWheelEvent, nullptr);
+  } else {
+    InitEvent(aWidget, aWheelEvent, &aLParam);
+  }
 
   aModKeyState.InitInputEvent(aWheelEvent);
 
@@ -800,8 +816,8 @@ bool MouseScrollHandler::LastEventInfo::InitWheelEvent(
   // Use orienter for computing our delta value with native delta value.
   int32_t orienter = mIsVertical ? -1 : 1;
 
-  aWheelEvent.mDeltaMode = mIsPage ? nsIDOMWheelEvent::DOM_DELTA_PAGE
-                                   : nsIDOMWheelEvent::DOM_DELTA_LINE;
+  aWheelEvent.mDeltaMode = mIsPage ? dom::WheelEvent_Binding::DOM_DELTA_PAGE
+                                   : dom::WheelEvent_Binding::DOM_DELTA_LINE;
 
   double& delta = mIsVertical ? aWheelEvent.mDeltaY : aWheelEvent.mDeltaX;
   int32_t& lineOrPageDelta = mIsVertical ? aWheelEvent.mLineOrPageDeltaY
@@ -818,7 +834,7 @@ bool MouseScrollHandler::LastEventInfo::InitWheelEvent(
   mAccumulatedDelta -=
       lineOrPageDelta * orienter * RoundDelta(nativeDeltaPerUnit);
 
-  if (aWheelEvent.mDeltaMode != nsIDOMWheelEvent::DOM_DELTA_LINE) {
+  if (aWheelEvent.mDeltaMode != dom::WheelEvent_Binding::DOM_DELTA_LINE) {
     // If the scroll delta mode isn't per line scroll, we shouldn't allow to
     // override the system scroll speed setting.
     aWheelEvent.mAllowToOverrideSystemScrollSpeed = false;
@@ -1317,11 +1333,15 @@ bool MouseScrollHandler::Device::Elantech::HandleKeyMessage(
                "%s command event",
                aWParam == VK_NEXT ? "Forward" : "Back"));
 
-      WidgetCommandEvent commandEvent(
-          true, nsGkAtoms::onAppCommand,
-          (aWParam == VK_NEXT) ? nsGkAtoms::Forward : nsGkAtoms::Back, aWidget);
-      InitEvent(aWidget, commandEvent);
-      aWidget->DispatchWindowEvent(&commandEvent);
+      WidgetCommandEvent appCommandEvent(
+          true, (aWParam == VK_NEXT) ? nsGkAtoms::Forward : nsGkAtoms::Back,
+          aWidget);
+
+      // In this scenario, the coordinate of the event isn't supplied, so pass
+      // nullptr as an argument to indicate using the coordinate from the last
+      // available window message.
+      InitEvent(aWidget, appCommandEvent, nullptr);
+      aWidget->DispatchWindowEvent(&appCommandEvent);
     } else {
       MOZ_LOG(gMouseScrollLog, LogLevel::Info,
               ("MouseScroll::Device::Elantech::HandleKeyMessage(): Consumed"));
