@@ -10,9 +10,11 @@
 #include "nsUXThemeData.h"
 #include "nsUXThemeConstants.h"
 #include "WinUtils.h"
+#include "mozilla/FontPropertyTypes.h"
 #include "mozilla/Telemetry.h"
 #include "mozilla/WindowsVersion.h"
 #include "gfxFontConstants.h"
+#include "gfxWindowsPlatform.h"
 
 using namespace mozilla;
 using namespace mozilla::widget;
@@ -38,7 +40,7 @@ LookAndFeel::OperatingSystemVersion nsLookAndFeel::GetOperatingSystemVersion() {
 
 static nsresult GetColorFromTheme(nsUXThemeClass cls, int32_t aPart,
                                   int32_t aState, int32_t aPropId,
-                                  nscolor &aColor) {
+                                  nscolor& aColor) {
   COLORREF color;
   HRESULT hr = GetThemeColor(nsUXThemeData::GetTheme(cls), aPart, aState,
                              aPropId, &color);
@@ -54,6 +56,38 @@ static int32_t GetSystemParam(long flag, int32_t def) {
   return ::SystemParametersInfo(flag, 0, &value, 0) ? value : def;
 }
 
+static nsresult SystemWantsDarkTheme(int32_t& darkThemeEnabled) {
+  if (!IsWin10OrLater()) {
+    darkThemeEnabled = 0;
+    return NS_OK;
+  }
+
+  nsresult rv = NS_OK;
+  nsCOMPtr<nsIWindowsRegKey> personalizeKey =
+      do_CreateInstance("@mozilla.org/windows-registry-key;1", &rv);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+
+  rv = personalizeKey->Open(
+      nsIWindowsRegKey::ROOT_KEY_CURRENT_USER,
+      NS_LITERAL_STRING(
+          "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Themes\\Personalize"),
+      nsIWindowsRegKey::ACCESS_QUERY_VALUE);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  uint32_t lightThemeEnabled;
+  rv = personalizeKey->ReadIntValue(NS_LITERAL_STRING("AppsUseLightTheme"),
+                                    &lightThemeEnabled);
+  if (NS_SUCCEEDED(rv)) {
+    darkThemeEnabled = !lightThemeEnabled;
+  }
+
+  return rv;
+}
+
 nsLookAndFeel::nsLookAndFeel()
     : nsXPLookAndFeel(),
       mUseAccessibilityTheme(0),
@@ -74,7 +108,8 @@ nsLookAndFeel::~nsLookAndFeel() {}
 
 void nsLookAndFeel::NativeInit() { EnsureInit(); }
 
-/* virtual */ void nsLookAndFeel::RefreshImpl() {
+/* virtual */
+void nsLookAndFeel::RefreshImpl() {
   nsXPLookAndFeel::RefreshImpl();
 
   for (auto e = mSystemFontCache.begin(), end = mSystemFontCache.end();
@@ -86,191 +121,191 @@ void nsLookAndFeel::NativeInit() { EnsureInit(); }
   mInitialized = false;
 }
 
-nsresult nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor) {
+nsresult nsLookAndFeel::NativeGetColor(ColorID aID, nscolor& aColor) {
   EnsureInit();
 
   nsresult res = NS_OK;
 
   int idx;
   switch (aID) {
-    case eColorID_WindowBackground:
+    case ColorID::WindowBackground:
       idx = COLOR_WINDOW;
       break;
-    case eColorID_WindowForeground:
+    case ColorID::WindowForeground:
       idx = COLOR_WINDOWTEXT;
       break;
-    case eColorID_WidgetBackground:
+    case ColorID::WidgetBackground:
       idx = COLOR_BTNFACE;
       break;
-    case eColorID_WidgetForeground:
+    case ColorID::WidgetForeground:
       idx = COLOR_BTNTEXT;
       break;
-    case eColorID_WidgetSelectBackground:
+    case ColorID::WidgetSelectBackground:
       idx = COLOR_HIGHLIGHT;
       break;
-    case eColorID_WidgetSelectForeground:
+    case ColorID::WidgetSelectForeground:
       idx = COLOR_HIGHLIGHTTEXT;
       break;
-    case eColorID_Widget3DHighlight:
+    case ColorID::Widget3DHighlight:
       idx = COLOR_BTNHIGHLIGHT;
       break;
-    case eColorID_Widget3DShadow:
+    case ColorID::Widget3DShadow:
       idx = COLOR_BTNSHADOW;
       break;
-    case eColorID_TextBackground:
+    case ColorID::TextBackground:
       idx = COLOR_WINDOW;
       break;
-    case eColorID_TextForeground:
+    case ColorID::TextForeground:
       idx = COLOR_WINDOWTEXT;
       break;
-    case eColorID_TextSelectBackground:
-    case eColorID_IMESelectedRawTextBackground:
-    case eColorID_IMESelectedConvertedTextBackground:
+    case ColorID::TextSelectBackground:
+    case ColorID::IMESelectedRawTextBackground:
+    case ColorID::IMESelectedConvertedTextBackground:
       idx = COLOR_HIGHLIGHT;
       break;
-    case eColorID_TextSelectForeground:
-    case eColorID_IMESelectedRawTextForeground:
-    case eColorID_IMESelectedConvertedTextForeground:
+    case ColorID::TextSelectForeground:
+    case ColorID::IMESelectedRawTextForeground:
+    case ColorID::IMESelectedConvertedTextForeground:
       idx = COLOR_HIGHLIGHTTEXT;
       break;
-    case eColorID_IMERawInputBackground:
-    case eColorID_IMEConvertedTextBackground:
+    case ColorID::IMERawInputBackground:
+    case ColorID::IMEConvertedTextBackground:
       aColor = NS_TRANSPARENT;
       return NS_OK;
-    case eColorID_IMERawInputForeground:
-    case eColorID_IMEConvertedTextForeground:
+    case ColorID::IMERawInputForeground:
+    case ColorID::IMEConvertedTextForeground:
       aColor = NS_SAME_AS_FOREGROUND_COLOR;
       return NS_OK;
-    case eColorID_IMERawInputUnderline:
-    case eColorID_IMEConvertedTextUnderline:
+    case ColorID::IMERawInputUnderline:
+    case ColorID::IMEConvertedTextUnderline:
       aColor = NS_SAME_AS_FOREGROUND_COLOR;
       return NS_OK;
-    case eColorID_IMESelectedRawTextUnderline:
-    case eColorID_IMESelectedConvertedTextUnderline:
+    case ColorID::IMESelectedRawTextUnderline:
+    case ColorID::IMESelectedConvertedTextUnderline:
       aColor = NS_TRANSPARENT;
       return NS_OK;
-    case eColorID_SpellCheckerUnderline:
+    case ColorID::SpellCheckerUnderline:
       aColor = NS_RGB(0xff, 0, 0);
       return NS_OK;
 
     // New CSS 2 Color definitions
-    case eColorID_activeborder:
+    case ColorID::Activeborder:
       idx = COLOR_ACTIVEBORDER;
       break;
-    case eColorID_activecaption:
+    case ColorID::Activecaption:
       idx = COLOR_ACTIVECAPTION;
       break;
-    case eColorID_appworkspace:
+    case ColorID::Appworkspace:
       idx = COLOR_APPWORKSPACE;
       break;
-    case eColorID_background:
+    case ColorID::Background:
       idx = COLOR_BACKGROUND;
       break;
-    case eColorID_buttonface:
-    case eColorID__moz_buttonhoverface:
+    case ColorID::Buttonface:
+    case ColorID::MozButtonhoverface:
       idx = COLOR_BTNFACE;
       break;
-    case eColorID_buttonhighlight:
+    case ColorID::Buttonhighlight:
       idx = COLOR_BTNHIGHLIGHT;
       break;
-    case eColorID_buttonshadow:
+    case ColorID::Buttonshadow:
       idx = COLOR_BTNSHADOW;
       break;
-    case eColorID_buttontext:
-    case eColorID__moz_buttonhovertext:
+    case ColorID::Buttontext:
+    case ColorID::MozButtonhovertext:
       idx = COLOR_BTNTEXT;
       break;
-    case eColorID_captiontext:
+    case ColorID::Captiontext:
       idx = COLOR_CAPTIONTEXT;
       break;
-    case eColorID_graytext:
+    case ColorID::Graytext:
       idx = COLOR_GRAYTEXT;
       break;
-    case eColorID_highlight:
-    case eColorID__moz_html_cellhighlight:
-    case eColorID__moz_menuhover:
+    case ColorID::Highlight:
+    case ColorID::MozHtmlCellhighlight:
+    case ColorID::MozMenuhover:
       idx = COLOR_HIGHLIGHT;
       break;
-    case eColorID__moz_menubarhovertext:
+    case ColorID::MozMenubarhovertext:
       if (!IsAppThemed()) {
         idx = nsUXThemeData::sFlatMenus ? COLOR_HIGHLIGHTTEXT : COLOR_MENUTEXT;
         break;
       }
       // Fall through
-    case eColorID__moz_menuhovertext:
+    case ColorID::MozMenuhovertext:
       if (mHasColorMenuHoverText) {
         aColor = mColorMenuHoverText;
         return NS_OK;
       }
       // Fall through
-    case eColorID_highlighttext:
-    case eColorID__moz_html_cellhighlighttext:
+    case ColorID::Highlighttext:
+    case ColorID::MozHtmlCellhighlighttext:
       idx = COLOR_HIGHLIGHTTEXT;
       break;
-    case eColorID_inactiveborder:
+    case ColorID::Inactiveborder:
       idx = COLOR_INACTIVEBORDER;
       break;
-    case eColorID_inactivecaption:
+    case ColorID::Inactivecaption:
       idx = COLOR_INACTIVECAPTION;
       break;
-    case eColorID_inactivecaptiontext:
+    case ColorID::Inactivecaptiontext:
       idx = COLOR_INACTIVECAPTIONTEXT;
       break;
-    case eColorID_infobackground:
+    case ColorID::Infobackground:
       idx = COLOR_INFOBK;
       break;
-    case eColorID_infotext:
+    case ColorID::Infotext:
       idx = COLOR_INFOTEXT;
       break;
-    case eColorID_menu:
+    case ColorID::Menu:
       idx = COLOR_MENU;
       break;
-    case eColorID_menutext:
-    case eColorID__moz_menubartext:
+    case ColorID::Menutext:
+    case ColorID::MozMenubartext:
       idx = COLOR_MENUTEXT;
       break;
-    case eColorID_scrollbar:
+    case ColorID::Scrollbar:
       idx = COLOR_SCROLLBAR;
       break;
-    case eColorID_threeddarkshadow:
+    case ColorID::Threeddarkshadow:
       idx = COLOR_3DDKSHADOW;
       break;
-    case eColorID_threedface:
+    case ColorID::Threedface:
       idx = COLOR_3DFACE;
       break;
-    case eColorID_threedhighlight:
+    case ColorID::Threedhighlight:
       idx = COLOR_3DHIGHLIGHT;
       break;
-    case eColorID_threedlightshadow:
+    case ColorID::Threedlightshadow:
       idx = COLOR_3DLIGHT;
       break;
-    case eColorID_threedshadow:
+    case ColorID::Threedshadow:
       idx = COLOR_3DSHADOW;
       break;
-    case eColorID_window:
+    case ColorID::Window:
       idx = COLOR_WINDOW;
       break;
-    case eColorID_windowframe:
+    case ColorID::Windowframe:
       idx = COLOR_WINDOWFRAME;
       break;
-    case eColorID_windowtext:
+    case ColorID::Windowtext:
       idx = COLOR_WINDOWTEXT;
       break;
-    case eColorID__moz_eventreerow:
-    case eColorID__moz_oddtreerow:
-    case eColorID__moz_field:
-    case eColorID__moz_combobox:
+    case ColorID::MozEventreerow:
+    case ColorID::MozOddtreerow:
+    case ColorID::Field:
+    case ColorID::MozCombobox:
       idx = COLOR_WINDOW;
       break;
-    case eColorID__moz_fieldtext:
-    case eColorID__moz_comboboxtext:
+    case ColorID::Fieldtext:
+    case ColorID::MozComboboxtext:
       idx = COLOR_WINDOWTEXT;
       break;
-    case eColorID__moz_dialog:
-    case eColorID__moz_cellhighlight:
+    case ColorID::MozDialog:
+    case ColorID::MozCellhighlight:
       idx = COLOR_3DFACE;
       break;
-    case eColorID__moz_win_accentcolor:
+    case ColorID::MozWinAccentcolor:
       if (mHasColorAccent) {
         aColor = mColorAccent;
       } else {
@@ -278,14 +313,14 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor) {
         aColor = NS_RGB(158, 158, 158);
       }
       return NS_OK;
-    case eColorID__moz_win_accentcolortext:
+    case ColorID::MozWinAccentcolortext:
       if (mHasColorAccentText) {
         aColor = mColorAccentText;
       } else {
         aColor = NS_RGB(0, 0, 0);
       }
       return NS_OK;
-    case eColorID__moz_win_mediatext:
+    case ColorID::MozWinMediatext:
       if (mHasColorMediaText) {
         aColor = mColorMediaText;
         return NS_OK;
@@ -293,7 +328,7 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor) {
       // if we've gotten here just return -moz-dialogtext instead
       idx = COLOR_WINDOWTEXT;
       break;
-    case eColorID__moz_win_communicationstext:
+    case ColorID::MozWinCommunicationstext:
       if (mHasColorCommunicationsText) {
         aColor = mColorCommunicationsText;
         return NS_OK;
@@ -301,17 +336,17 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor) {
       // if we've gotten here just return -moz-dialogtext instead
       idx = COLOR_WINDOWTEXT;
       break;
-    case eColorID__moz_dialogtext:
-    case eColorID__moz_cellhighlighttext:
+    case ColorID::MozDialogtext:
+    case ColorID::MozCellhighlighttext:
       idx = COLOR_WINDOWTEXT;
       break;
-    case eColorID__moz_dragtargetzone:
+    case ColorID::MozDragtargetzone:
       idx = COLOR_HIGHLIGHTTEXT;
       break;
-    case eColorID__moz_buttondefault:
+    case ColorID::MozButtondefault:
       idx = COLOR_3DDKSHADOW;
       break;
-    case eColorID__moz_nativehyperlinktext:
+    case ColorID::MozNativehyperlinktext:
       idx = COLOR_HOTLIGHT;
       break;
     default:
@@ -326,7 +361,7 @@ nsresult nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor) {
   return res;
 }
 
-nsresult nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult) {
+nsresult nsLookAndFeel::GetIntImpl(IntID aID, int32_t& aResult) {
   nsresult res = nsXPLookAndFeel::GetIntImpl(aID, aResult);
   if (NS_SUCCEEDED(res)) return res;
   res = NS_OK;
@@ -440,7 +475,7 @@ nsresult nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult) {
       res = NS_ERROR_NOT_IMPLEMENTED;
       break;
     case eIntID_DWMCompositor:
-      aResult = nsUXThemeData::CheckForCompositor();
+      aResult = gfxWindowsPlatform::GetPlatform()->DwmCompositionEnabled();
       break;
     case eIntID_WindowsAccentColorInTitlebar: {
       nscolor unused;
@@ -471,7 +506,8 @@ nsresult nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult) {
     } break;
     case eIntID_WindowsGlass:
       // Aero Glass is only available prior to Windows 8 when DWM is used.
-      aResult = (nsUXThemeData::CheckForCompositor() && !IsWin8OrLater());
+      aResult = (gfxWindowsPlatform::GetPlatform()->DwmCompositionEnabled() &&
+                 !IsWin8OrLater());
       break;
     case eIntID_AlertNotificationOrigin:
       aResult = 0;
@@ -544,6 +580,27 @@ nsresult nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult) {
     case eIntID_ContextMenuOffsetHorizontal:
       aResult = 2;
       break;
+    case eIntID_SystemUsesDarkTheme:
+      res = SystemWantsDarkTheme(aResult);
+      break;
+    case eIntID_PrefersReducedMotion: {
+      BOOL enableAnimation = TRUE;
+      ::SystemParametersInfoW(SPI_GETCLIENTAREAANIMATION, 0, &enableAnimation,
+                              0);
+      aResult = enableAnimation ? 0 : 1;
+      break;
+    }
+    case eIntID_PrimaryPointerCapabilities: {
+      PointerCapabilities caps =
+          widget::WinUtils::GetPrimaryPointerCapabilities();
+      aResult = static_cast<int32_t>(caps);
+      break;
+    }
+    case eIntID_AllPointerCapabilities: {
+      PointerCapabilities caps = widget::WinUtils::GetAllPointerCapabilities();
+      aResult = static_cast<int32_t>(caps);
+      break;
+    }
     default:
       aResult = 0;
       res = NS_ERROR_FAILURE;
@@ -551,7 +608,7 @@ nsresult nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult) {
   return res;
 }
 
-nsresult nsLookAndFeel::GetFloatImpl(FloatID aID, float &aResult) {
+nsresult nsLookAndFeel::GetFloatImpl(FloatID aID, float& aResult) {
   nsresult res = nsXPLookAndFeel::GetFloatImpl(aID, aResult);
   if (NS_SUCCEEDED(res)) return res;
   res = NS_OK;
@@ -571,8 +628,8 @@ nsresult nsLookAndFeel::GetFloatImpl(FloatID aID, float &aResult) {
 }
 
 static bool GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
-                           nsString &aFontName, gfxFontStyle &aFontStyle) {
-  const LOGFONTW *ptrLogFont = nullptr;
+                           nsString& aFontName, gfxFontStyle& aFontStyle) {
+  const LOGFONTW* ptrLogFont = nullptr;
   LOGFONTW logFont;
   NONCLIENTMETRICSW ncm;
   char16_t name[LF_FACESIZE];
@@ -665,15 +722,15 @@ static bool GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
   aFontStyle.size = pixelHeight;
 
   // FIXME: What about oblique?
-  aFontStyle.style =
-      (ptrLogFont->lfItalic) ? NS_FONT_STYLE_ITALIC : NS_FONT_STYLE_NORMAL;
+  aFontStyle.style = (ptrLogFont->lfItalic) ? FontSlantStyle::Italic()
+                                            : FontSlantStyle::Normal();
 
   // FIXME: Other weights?
-  aFontStyle.weight = (ptrLogFont->lfWeight == FW_BOLD ? NS_FONT_WEIGHT_BOLD
-                                                       : NS_FONT_WEIGHT_NORMAL);
+  aFontStyle.weight = (ptrLogFont->lfWeight == FW_BOLD ? FontWeight::Bold()
+                                                       : FontWeight::Normal());
 
   // FIXME: Set aFontStyle->stretch correctly!
-  aFontStyle.stretch = NS_FONT_STRETCH_NORMAL;
+  aFontStyle.stretch = FontStretch::Normal();
 
   aFontStyle.systemFont = true;
 
@@ -687,10 +744,9 @@ static bool GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
   return true;
 }
 
-bool nsLookAndFeel::GetFontImpl(FontID anID, nsString &aFontName,
-                                gfxFontStyle &aFontStyle,
-                                float aDevPixPerCSSPixel) {
-  CachedSystemFont &cacheSlot = mSystemFontCache[anID];
+bool nsLookAndFeel::GetFontImpl(FontID anID, nsString& aFontName,
+                                gfxFontStyle& aFontStyle) {
+  CachedSystemFont& cacheSlot = mSystemFontCache[anID];
 
   bool status;
   if (cacheSlot.mCacheValid) {
@@ -711,9 +767,6 @@ bool nsLookAndFeel::GetFontImpl(FontID anID, nsString &aFontName,
       cacheSlot.mFontStyle = aFontStyle;
     }
   }
-  // now convert the logical font size from GetSysFontInfo into device pixels
-  // for layout
-  aFontStyle.size *= aDevPixPerCSSPixel;
   return status;
 }
 
@@ -744,7 +797,7 @@ nsTArray<LookAndFeelInt> nsLookAndFeel::GetIntCacheImpl() {
 }
 
 void nsLookAndFeel::SetIntCacheImpl(
-    const nsTArray<LookAndFeelInt> &aLookAndFeelIntCache) {
+    const nsTArray<LookAndFeelInt>& aLookAndFeelIntCache) {
   for (auto entry : aLookAndFeelIntCache) {
     switch (entry.id) {
       case eIntID_UseAccessibilityTheme:
@@ -760,7 +813,8 @@ void nsLookAndFeel::SetIntCacheImpl(
   }
 }
 
-/* static */ nsresult nsLookAndFeel::GetAccentColor(nscolor &aColor) {
+/* static */
+nsresult nsLookAndFeel::GetAccentColor(nscolor& aColor) {
   nsresult rv;
 
   if (!mDwmKey) {
@@ -794,7 +848,8 @@ void nsLookAndFeel::SetIntCacheImpl(
   return rv;
 }
 
-/* static */ nsresult nsLookAndFeel::GetAccentColorText(nscolor &aColor) {
+/* static */
+nsresult nsLookAndFeel::GetAccentColorText(nscolor& aColor) {
   nscolor accentColor;
   nsresult rv = GetAccentColor(accentColor);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -857,4 +912,6 @@ void nsLookAndFeel::EnsureInit() {
     DWORD color = ::GetSysColor(i);
     mSysColorTable[i - SYS_COLOR_MIN] = COLOREF_2_NSRGB(color);
   }
+
+  RecordTelemetry();
 }
